
                   
define([
    'dojo/dom-style',
    'sna/applicationComponents/NetworkView/util/NLDHelper',
    'sna/applicationComponents/NetworkView/util/CustomMeasuresHelper',
    'dojox/lang/functional',
    'dojo/_base/array',
    'dojo/_base/lang',
    'sna/applicationComponents/NetworkView/util/NetworkStatefulHelper',
    'dojo/Deferred',
    'sna/data/NetworkData',
    "commons/geom/Point",
    'sna/applicationComponents/NetworkView/NetworkMapModel',
    'dojo/_base/declare'
], function(domStyle, NLDHelper, CustomMeasuresHelper, functional, array, lang,
             NetworkStatefulHelper, Deferred, networkData, Point, NetworkMapModel, declare) {
    return declare(null, {
        
        constructor: function(args) {
            this.networkFacetController = args.networkFacetController;
            this.networkView = args.networkView;
        },

        _removeLinksOfNode: function(node, linksArray) {
            var newLinksArray = [];

            array.forEach(linksArray, function(link) {
                if(link.fromNodeId !== node.uid && link.toNodeId !== node.uid) {
                    newLinksArray.push(link);
                }
            });

            return newLinksArray;
        },

        _pruneNodesFromDiagram: function(linksArray, nodesArray, unchartedNodesArray) {
            var that = this;
            var newLinksArray = linksArray;

            array.forEach(unchartedNodesArray, function(node) {
                newLinksArray = that._removeLinksOfNode(node, newLinksArray);
            });

            this.model.oldLinkList = linksArray;

            var networkView = that.networkView;
            networkView.networkState.links = newLinksArray;

            NetworkStatefulHelper.clear();
            NetworkStatefulHelper.sync(nodesArray.concat(newLinksArray));
        },

        start: function() {
            var that = this;
            require([
                'sna/applicationComponents/NetworkView/NetworkMapModel',
                'sna/applicationComponents/NetworkView/uncharted/UnchartedNodesController'
            ], function( NetworkMapModel, UnchartedNodesController ) {
                that.unchartedNodesController = new UnchartedNodesController({
                    nldAside: that.networkView.nldAside,
                    vertical: that.networkView.vertical,
                    bundlePrefix: that.networkView.bundlePrefix
                });
                that.model = new NetworkMapModel({
                    controller: that,
                    networkView: that.networkView,
                    NLDHelper: NLDHelper
                });

                that.model.start();
            });
        },

        getMap: function( ) {
            return this.model.map;
        },

        hide: function() {          
            var networkView = this.networkView;
            networkView.networkState.nodes = this.model.oldNodeList;
            networkView.networkState.links = this.model.oldLinkList;
            networkView.networkState.groups = this.model.oldGroupList;

            var that = this;

            
            NetworkStatefulHelper.clear();
            var facets = this.networkFacetController.facets;

            NetworkStatefulHelper.sync(networkView.networkState.nodes.concat(networkView.networkState.links.concat(networkView.networkState.groups)));
            this.unchartedNodesController.hideLink();
            NetworkStatefulHelper.handleRawGroups(networkView.networkState);

            CustomMeasuresHelper.performAspectTracing(networkView.networkState, facets, function( entity, highlight ) {
                networkView.aspectTracingAction(entity, highlight);
            });
        },

        updateMenuState: function() {
            this.model.updateMenuState();
        },

        show: function() {
            this.doMaps();
        },

        getNodes: function() {
            var deferred = new Deferred();

            var that = this;
            var poller = function() {
                var networkView = that.networkView;
                if(networkView.networkState && (networkView.networkState.nodes.length > 0)) {
                    var nodes = networkView.networkState.nodes;
                    var nodesOnMapExist = false;

                    for(var i = 0; i < nodes.length; i += 1) {
                        var node = nodes[i];

                        var latitude = node.evolutions2[0].latitude;
                        var longitude = node.evolutions2[0].longitude;

                        if (latitude == null || longitude == null || typeof(latitude) === 'undefined' || typeof(longitude) === 'undefined') {
                            continue;
                        }

                        nodesOnMapExist = true;
                        break;
                    }

                    deferred.resolve(nodesOnMapExist);
                }

                setTimeout(poller, 100);
            };
            poller();

            return deferred.promise;
        },

        doMaps: function() {
            var networkView = this.networkView;

            var nodeList = networkView.networkState.nodes;
            var mapNodeList = new Array();
            var firstNode = nodeList[1];
            var that = this;
            var latitude;
            var longitude;
            var bounds = new Array();

            networkView.networkState.unchartedNodes = new Array();

            for (var i = 0; i < nodeList.length; i++) {
                var node = nodeList[i];
                latitude = node.evolutions2[0].latitude;
                longitude = node.evolutions2[0].longitude;

                if(typeof(latitude) === 'undefined' || typeof(longitude) === 'undefined') {
                    node.originalNodeIndex = i;
                    networkView.networkState.unchartedNodes.push(node);
                    continue;
                }

                mapNodeList[mapNodeList.length] = node;
                if (latitude != null && longitude != null) {
                    bounds[bounds.length]= new L.LatLng(latitude,longitude);
                }
            }

            
            networkView.NetworkStatefulHelper.reverseSync(networkView.networkState.nodes);

            
            this.model.oldNodeList = lang.clone(networkView.networkState.nodes);
            this.model.oldGroupList = lang.clone(networkView.networkState.groups);
            networkView.networkState.nodes = mapNodeList;

            var theObject = null;

            this.model.show( networkData.getOSMUrl(this.networkView.networkState.vertical), bounds, networkView.networkState.unchartedNodes);
            this.model.showPenaltyBox();
            this.unchartedNodesController.start(networkView.networkState.unchartedNodes);

            
            
            this._pruneNodesFromDiagram(networkView.networkState.links, networkView.networkState.nodes, networkView.networkState.unchartedNodes);

            
            
            functional.forIn(networkView.networkState.nodes, function( node ) {
                networkView.updateEntity(node.uid, {
                    highlight: node.highlight
                });
            });
        },

        calcXY: function() {
            var networkView = this.networkView;
            var map = this.model.map;
            var nodeList = networkView.networkState.nodes;
            var adjustmentAmount = map._size.y - domStyle.get(this.networkView.snadiagramcontentpane.domNode,"height");

            for (var i = 0; i < nodeList.length; i++) {
                var node = nodeList[i];

                if (node === null)
                    continue;

                var latitude = node.evolutions2[0].latitude;
                var longitude = node.evolutions2[0].longitude;

                if (latitude == null || longitude == null || typeof(latitude) === 'undefined' || typeof(longitude) === 'undefined')
                    continue;

                var point = map.latLngToContainerPoint(new L.LatLng(latitude, longitude));
                var myNode = document.getElementById(node.uid+"myNode");
                var translatedPoint = NLDHelper.localToData(new Point(point.x, map._size.y - point.y - adjustmentAmount));

                var nodeUid = 'bothChartAndTableProperties';
                networkView.updateEntity(node.uid, {
                    positionX: translatedPoint.x,
                    positionY: translatedPoint.y
                });
            }

        },

        isOpen: function() {
            var mapContainer = this.networkView.mapcontainer;
            return mapContainer.children.length > 0 && domStyle.get(mapContainer, 'display') !== 'none';
        }
    });
});
               