
                   
define([
    'dojo/aspect',
    'dojo/Stateful',
    'sna/model/ApplicationContextModel',
    'sna/SessionTimeoutMonitor/SessionTimeoutMonitor',
    'fraudCommons/widgets/LoadingSpinner/LoadingSpinner',
    'dojo/_base/array',
    'dijit/layout/BorderContainer',
    'fraudCommons/util/GUIDUtil',
    'sna/util/PluginUtil',
    'sna/util/ContextPreferenceUtil',
    'sna/data/UserData',
    'fraudCommons/util/JSONUtil',
    'dojo/query',
    'sna/util/I18nUtil',
    'dijit/form/Button',
    'sna/util/DebugUtil',
    'dojo/dom-construct',
    'sna/util/MessageUtil',
    'dojo/_base/lang',
    'sna/data/NetworkData',
    'dojox/lang/functional',
    'dijit/_TemplatedMixin',
    'dijit/_WidgetsInTemplateMixin',
    'dojo/_base/declare',
    'dojo/text!./templates/NetworkView.html'
], function (aspect, Stateful, ApplicationContextModel, SessionTimeoutMonitor, LoadingSpinner, array, BorderContainer, GUIDUtil, PluginUtil,
             ContextPreferenceUtil, UserData, JSONUtil,
             query, I18nUtil, Button, DebugUtil, domConstruct, MessageUtil,
             lang, networkData, functional, TemplatedMixin, WidgetsInTemplateMixin,
             declare, template) {

    var userPreferenceKey = 'networkgraph';
    var map;
    var divmap;
    var dialog;

    
    var PrototypeEntity = {
        emptyNode: {
            "uid": "",
            "nodeLabel": "",
            "nodeColor": "",
            "nodeType": "",
            "nodeSymbol": "",
            "nodeToolTip": "",
            "isNodeHidden": false,
            "nodeDegrees": 0,
            "borderSize": 1,
            "type": "NODE",
            "evolves": false,
            "evolutions2": {},
            "isDirty": false,
            "customMeasures": {}
        },

        emptyLink: {
            clusterId: "",
            fromNodeId: "",
            toNodeId: "",
            linkWidth: 2,
            linkStyle: "1",
            linkColor: "0x000000",
            linkLabel: "",
            linkStartDt: "", 
            linkEndDt: "", 
            type: "LINK",
            strength: 20,
            style: "A"
        },

        emptyGroup: {
            uid: "",
            nodeUid: "",
            groupColor: "0x00cc00",
            groupSymbol: "User_2",
            isNodeHidden: false,
            borderSize: 2,
            type: "GROUP"
        }
    };

    var getEntityFromUID = function (uid) {
        var entity = null;

        
        functional.forIn(this.networkState.nodes, function (node, index) {
            if (node.uid === uid) {
                entity = node;
                return;
            }
        });

        
        if (!entity) {
            functional.forIn(this.networkState.links, function (link, index) {
                if (link.uid === uid) {
                    entity = link;
                    return;
                }
            });
        }

        
        if (!entity) {
            functional.forIn(this.networkState.groups, function (group, index) {
                if (group.uid === uid) {
                    entity = group;
                    return;
                }
            });
        }

        return entity;
    };

    
    var getCurrentlyFocusedEntity = function () {
        var currentlyFocusedEntity = undefined;

        var that = this;
        var currentlyFocusedUid = that.networkState.focusedEntityId;

        
        functional.forIn(this.networkState.nodes, function (node, index) {
            if (node.uid === currentlyFocusedUid) {
                currentlyFocusedEntity = node;
                return;
            }
        });

        
        if (!currentlyFocusedEntity) {
            functional.forIn(this.networkState.links, function (link, index) {
                if (link.uid === currentlyFocusedUid) {
                    currentlyFocusedEntity = link;
                    return;
                }
            });
        }

        
        if (!currentlyFocusedEntity) {
            functional.forIn(this.networkState.groups, function (group, index) {
                if (group.uid === currentlyFocusedUid) {
                    currentlyFocusedEntity = group;
                    return;
                }
            });
        }

        return currentlyFocusedEntity || {};
    };

    
    var updateTimeSlider = function () {
        var timeSliderValues = this.TimeSliderHelper.getTimeSliderValues(this.networkState);
        this.model.updateTimeSlider(timeSliderValues);
    };

    
    var removeSingleGroups = function (groups) {
        var newGroups = new Array();
        for (var i = 0; i < groups.length; i++) {
            for (var j = 0; j < groups.length; j++) {
                if (groups[i].uid === groups[j].uid && i != j) {
                    newGroups.push(groups[i]);
                    break;
                }
            }
        }
        return newGroups;
    };

    
    var isSaveNetworkAllowedAction = function (isSaveNetworkAllowed) {
        ApplicationContextModel.set('unsavedChanges', false);

        var saveButton = this.snaSaveButton;
        if(!saveButton) return;
        if( isSaveNetworkAllowed === true ) {
            saveButton.set('disabled', false);
        }
        else {
            saveButton.set('disabled', true);
        }
    };

    var doesLinkAlreadyExist = function (node1, node2) {
        var alreadyExists = false;

        var nodeUid1 = node1.uid;
        var nodeUid2 = node2.uid;

        functional.forIn(this.networkState.links, function (link, linkIndex) {
            var fromNodeId = link.fromNodeId;
            var toNodeId = link.toNodeId;

            var fromNodeIdMatches = (fromNodeId === nodeUid1 || fromNodeId === nodeUid2);
            var toNodeIdMatches = (toNodeId === nodeUid1 || toNodeId === nodeUid2);

            if (fromNodeIdMatches && toNodeIdMatches) {
                alreadyExists = true;
            }
        });

        return alreadyExists;
    };

    var arrayMinusUid = function (array, field, uids) {
        var subsetArray = [];

        functional.forIn(array, function (entity, index) {
            var duplicate = uids.some(function (uid) {
                return uid === entity[field];
            });

            if (!duplicate) {
                subsetArray.push(entity);
            }
        });

        return subsetArray;
    };

    var getNodesAndLinks = function (entities) {
        var network = {"nodes": [], "links": [], "groups": []};
        array.forEach(entities, function (entity, index) {
            var entityType = entity.type._id;
            if (entityType === 'NODE') {
                network.nodes.push(entity);
            } else if (entityType === 'LINK') {
                network.links.push(entity);
            }
        });
        return network;
    };

    
    var getNodesLinksAndGroups = function (entities) {
        var network = {"nodes": [], "links": [], "groups": []};
        array.forEach(entities, function (entity, index) {
            var entityType = entity.type._id;
            if (entityType === 'NODE') {
                network.nodes.push(entity);
            } else if (entityType === 'LINK') {
                network.links.push(entity);
            } else if (entityType === 'GROUP') {
                network.groups.push(entity);
            }
        });
        return network;
    };

    return declare([BorderContainer, TemplatedMixin, WidgetsInTemplateMixin], {
        templateString: template,

        tabId: 'networkView',

        constructor: function (args) {
            this.facade = args.facade;
            this.mode = this.MODES.FULL_SNA;

            this.vertical = args.vertical;
            this.bundlePrefix = args.bundlePrefix;
            this.entity = args.entity;
            this.gutters = false;
            this.loading = new Stateful({
                loading: false
            });
            this.supportLinkDetails = ContextPreferenceUtil.getPreference("support_link_details_flg", this.vertical) === 'y';
        },

        startup: function () {
            this.inherited(arguments);
        },

        postCreate: function () {
            this.inherited(arguments);
            this.networkviewfacetspanel.set('title', I18nUtil.get('networkview_filters'));
            this.networkviewpropertiesscontentpane.set('title', I18nUtil.get('networkview_properties'));
            this.snamainbordercontainer.removeChild(this.snadetailscontentpane);
        },

        MODES: {
            'FULL_SNA': 0,
            'SNALITE': 1
        },

        
        setMode: function (mode) {
            
            if (mode === 'SNALITE') {
                this.mode = this.MODES.SNALITE;
            }
            else {
                this.mode = this.MODES.FULL_SNA;
            }
        },

        
        updateEntity: function (uid, propertyMap, reverseSyncUse) {
            var that = this;
            var getJsonEntity = function (entityUid) {
                var entity = null;

                
                var nodes = that.networkState.nodes;
                for (var i = 0; i < nodes.length; i++) {
                    if (nodes[i] && nodes[i].uid === entityUid) {
                        entity = nodes[i];
                        break;
                    }
                }

                if (!entity) {
                    
                    var links = that.networkState.links;
                    for (var i = 0; i < links.length; i++) {
                        if (links[i] && links[i].uid === entityUid) {
                            entity = links[i];
                            break;
                        }
                    }
                }

                if (!entity) {
                    
                    var groups = that.networkState.groups;
                    for (var i = 0; i < groups.length; i++) {
                        if (groups[i] && groups[i].uid === entityUid) {
                            entity = groups[i];
                            break;
                        }
                    }
                }

                return entity || {};
            };

            var jsonEntity = getJsonEntity(uid);

            lang.mixin(jsonEntity, propertyMap);

            
            
            
            
            if (reverseSyncUse === 'useReverseSync') {
                this.NetworkStatefulHelper.reverseSync([jsonEntity]);
            }
            this.NetworkStatefulHelper.sync([jsonEntity]);
        },

        
        start: function (callback) {
            var that = this;
            that.loading.set('loading', true);
            require([
                'sna/applicationComponents/NetworkView/NetworkModel',
                'sna/applicationComponents/NetworkView/util/TimeSliderHelper',
                'sna/applicationComponents/NetworkView/util/NodeLinkFilterHelper',
                'sna/applicationComponents/NetworkView/util/CustomMeasuresHelper',
                'sna/applicationComponents/NetworkView/util/NLDHelper',
                'sna/applicationComponents/NetworkView/widgets/LayoutError/LayoutError',
                'sna/applicationComponents/NetworkView/util/NetworkStatefulHelper',
                'sna/applicationComponents/NetworkView/util/NetworkDataPreProcessor',
                'sna/applicationComponents/NetworkView/util/CollapseExpandedNodesHelper',
                'sna/applicationComponents/NetworkView/util/UndoStackHelper',
                'sna/applicationComponents/NetworkView/properties/NetworkPropertiesController',
                'sna/applicationComponents/NetworkView/NetworkFacetController',
                'sna/applicationComponents/NetworkView/NetworkMapController',
                'sna/applicationComponents/NetworkView/details/NetworkDetailsController'
            ], function (NetworkModel, TimeSliderHelper, NodeLinkFilterHelper, CustomMeasuresHelper, NLDHelper,
                         LayoutError, NetworkStatefulHelper, NetworkDataPreProcessor, CollapseExpandedNodesHelper, UndoStackHelper,
                         NetworkPropertiesController, NetworkFacetController, NetworkMapController, NetworkDetailsController) {
                networkData.linkPreProcessor = NetworkDataPreProcessor.preprocessLinks;
                networkData.nodePreProcessor = NetworkDataPreProcessor.preprocessNodes;

                this.sessionTimeoutMonitor = new SessionTimeoutMonitor({
                    vertical: that.vertical
                });

                if (DebugUtil.isSNALiteMode()) {
                    this.sessionTimeoutMonitor.start();
                }

                that.TimeSliderHelper = TimeSliderHelper;
                that.NodeLinkFilterHelper = NodeLinkFilterHelper;
                that.CustomMeasuresHelper = CustomMeasuresHelper;
                that.CollapseExpandedNodesHelper = CollapseExpandedNodesHelper;
                that.UndoStackHelper = UndoStackHelper;
                that.NLDHelper = NLDHelper;
                that.NetworkStatefulHelper = NetworkStatefulHelper;
                
                var vertical = that.vertical;

                
                var entity = that.entity;

                that.networkPropertiesController = new NetworkPropertiesController({
                    networkView: that
                });
                that.networkFacetController = new NetworkFacetController({
                    networkView: that
                });
                that.networkMapController = new NetworkMapController({
                    networkFacetController: that.networkFacetController,
                    networkView: that
                });

                that.networkDetailsController = new NetworkDetailsController({
                    networkView: that
                });

                that.UndoStackHelper.clearStack(that.UndoStackHelper.GROW, that);
                that.UndoStackHelper.clearStack(that.UndoStackHelper.REGROUP, that);

                dialog = new LayoutError({});

                
                that.model = new NetworkModel({
                    controller: that
                });

                aspect.after(that.snamainbordercontainer, "layout", lang.hitch(that.model, that.model.refreshView));

                
                
                
                var contentPane = that.networkviewpropertiesscontentpane;
                aspect.after(contentPane, "expand", that.model.shiftNetworkLeft);
                aspect.before(contentPane, "collapse", that.model.shiftNetworkRight);

                if (that.model.isAlreadyLoaded() === true) {
                    return;
                }

                
                that.networkState = {
                    vertical: vertical,
                    entity: entity,
                    nodes: {},
                    links: {},
                    facets: {},
                    groups: [],
                    unchartedNodes: [],
                    focusedEntityId: -1,
                    actionableEntityId: entity.actionableEntityId,
                    actionableEntityName: entity.actionableEntityName,
                    actionableEntityType: entity.actionableEntityType
                };

                
                that.model.load(function () {
                    
                    that.networkPropertiesController.start();
                    that.networkDetailsController.start();

                    
                    var preferredSource = 'NETWORK_SOURCE_DATABASE';
                    

                    var loadingComplete = false;
                    var completionFunction = function () {
                        return loadingComplete;
                    };

                    new LoadingSpinner().target(that.snamaincontentpane).until(completionFunction).startSpinners(function () {
                    });

                    
                    
                    
                    if(preferredSource == "NETWORK_SOURCE_DATABASE") {
                        networkData.checkForSavedNetwork(vertical, that.networkState.entity.actionableEntityId, that.networkState.entity.actionableEntityType, function (savedNetworkExists) {
                                if(savedNetworkExists === "true") {
                                    
                                    getNetworkAndFacetsFromServer("NETWORK_SOURCE_STOREDPROCESS", function (stpData) {
                                        var colorModel = stpData.colorModel;

                                        
                                        getNetworkAndFacetsFromServer("NETWORK_SOURCE_DATABASE", function (results) {
                                            results.colorModel = colorModel;
                                            processNetworkAndFacetsFromServer(results);
                                        });
                                    });
                                }else{
                                    getNetworkAndFacetsFromServer("NETWORK_SOURCE_STOREDPROCESS", processNetworkAndFacetsFromServer);
                                }
                            });
                    }
                    else{
                        getNetworkAndFacetsFromServer("NETWORK_SOURCE_STOREDPROCESS", processNetworkAndFacetsFromServer);
                    }

                    function processNetworkAndFacetsFromServer(results){
                        UserData.getUser( function( userData ) {
                            loadingComplete = true;

                            if(results.error && results.error.code === 'error_unauthorized') {
                                MessageUtil.showErrorMessage(I18nUtil.get('applicationcomponents.networkView.unauthorized'), false);

                                array.forEach(that.networktoolbar.getChildren(), function(child) {
                                    child.set('disabled', true);
                                });

                                return;
                            }

                            lang.mixin(that.networkState, results);

                            lang.hitch(that, isSaveNetworkAllowedAction, results.isSaveNetworkAllowed)();

                            
                            functional.forEach(results.nodes, function(node, index) {
                                if(node.borderSize > 20) {
                                    node.borderSize = 20;
                                }
                            });
                            var justNodesAndLinks = results.nodes;
                            justNodesAndLinks = justNodesAndLinks.concat(results.links);
                            if (results.groups != null) {
                                results.groups = removeSingleGroups(results.groups);
                                justNodesAndLinks = justNodesAndLinks.concat(results.groups);
                            }
                            that.NetworkStatefulHelper.clear();
                            that.NetworkStatefulHelper.sync(justNodesAndLinks);

                            that.model.render(that.NetworkStatefulHelper.getStatefulEntityArray(), results.plugins, vertical, function() {
                                
                                var timeSliderValues = that.TimeSliderHelper.getTimeSliderValues(that.networkState);
                                that.model.setupTimeSlider(timeSliderValues);

                                
                                that.model.plugins = PluginUtil.createPluginsFromMap(results.plugins, {
                                    component: that,
                                    networkState: that.networkState
                                }, vertical);

                                that.updateEntitiesFromTimesliderValue();
                                that.filterDataFromTimeSliderValue(that.model.getTimeSliderValue());

                                that.NetworkStatefulHelper.handleRawGroups(results);

                                
                                that.networkFacetController.start(that.networkState.facets);
                                that.networkFacetController.refresh(that.networkState.facets);
                                that.networkMapController.start();

                                that.loading.set('loading', false);
                            });
                        });
                        callback();
                    }

                    function getNetworkAndFacetsFromServer(preferredSource, callback){
                        networkData.getSocialNetworkAndFacets(vertical, that.networkState.entity.actionableEntityId, that.networkState.entity.actionableEntityType, preferredSource, callback);
                    }


                    if( DebugUtil.isShowNetworkInfoMode() ) {
                        var networktoolbar = that.networktoolbar;
                        networktoolbar.addChild(new Button({
                            id: 'snaShowNetworkInfoButton',
                            iconClass: 'sasPropertiesIcon',
                            showLabel: false,
                            label: I18nUtil.get('networkview_shownetworkinformation')
                        }));
                    }
                });
            });
        },


        getSelectedEntities: function () {
            var currentlySelectedIndicies = this.NLDHelper.getCurrentlySelectedIndices();
            return currentlySelectedEntities = this.NetworkStatefulHelper.entitiesFromIndicies(currentlySelectedIndicies);
        },

        getUnselectedEntities: function () {
            return this.NetworkStatefulHelper.entitiesFromIndicies(this.NLDHelper.getUnselectedIndices());
        },


        
        startAddNodeActions: function () {
            var that = this;

            var currentlySelectedEntities = this.getSelectedEntities();

            if (currentlySelectedEntities.length !== 1 || currentlySelectedEntities[0].type._id !== 'NODE') {
                MessageUtil.showErrorMessage(I18nUtil.get('networkview_youneedtohaveanodeselected'));
            }
            else {
                var currentlyFocusedEntity = currentlySelectedEntities[0];

                var newPrototypeNode = {};
                lang.mixin(newPrototypeNode, PrototypeEntity.emptyNode);

                var newPrototypeLink = {};
                lang.mixin(newPrototypeLink, PrototypeEntity.emptyLink);

                this.model.showAddNodeDialog(this.networkState, currentlyFocusedEntity, newPrototypeNode, newPrototypeLink, function (newNode, newLink) {
                    that.networkState.nodes.push(newNode);
                    that.networkState.links.push(newLink);

                    that.NetworkStatefulHelper.sync([newNode, newLink]);
                    that.NodeLinkFilterHelper.incrementNodeDegrees(newLink);

                    MessageUtil.showSuccessMessage(I18nUtil.get('networkview_nodeadded'));
                    lang.hitch(that, updateTimeSlider)();

                    
                    that.UndoStackHelper.clearStackIfContains(that.UndoStackHelper.GROW, [
                        currentlyFocusedEntity.uid
                    ], that);

                    
                    that.networkMapController.updateMenuState();
                });
            }
        },

        
        startEditEntityActions: function () {
            var that = this;

            var currentlySelectedEntities = this.getSelectedEntities();

            if (currentlySelectedEntities.length === 0) {
                MessageUtil.showErrorMessage(I18nUtil.get('networkview_selectanentity'));
            }
            else if (currentlySelectedEntities.length > 1) {
                MessageUtil.showErrorMessage(I18nUtil.get('networkview_pleaseselectonlyoneentitytoedit'));
            }
            else if (currentlySelectedEntities.length === 1) {
                var currentlyFocusedEntity = currentlySelectedEntities[0];
                var isNode = currentlyFocusedEntity.type._id === 'NODE';
                var isLink = currentlyFocusedEntity.type._id === 'LINK';

                var jsonEntity = lang.hitch(this, getEntityFromUID, currentlyFocusedEntity.get('uid'))();

                if (isNode === true) {
                    this.model.showEditNodeDialog(jsonEntity, function (updatedNode) {
                        
                        that.NetworkStatefulHelper.reverseSync(that.networkState.nodes);

                        that.NodeLinkFilterHelper.updateNode(jsonEntity, updatedNode);

                        MessageUtil.showSuccessMessage(I18nUtil.get('networkview_nodeupdated'));
                        lang.hitch(that, updateTimeSlider)();
                        that.updateEntitiesFromTimesliderValue();
                    });
                }
                else if (isLink === true) {
                    this.model.showEditLinkDialog(this.networkState, jsonEntity, function (updatedLink) {
                        
                        that.NetworkStatefulHelper.reverseSync(that.networkState.nodes);

                        MessageUtil.showSuccessMessage(I18nUtil.get('networkview_linkupdated'));
                        that.NetworkStatefulHelper.sync([updatedLink]);
                        lang.hitch(that, updateTimeSlider)();
                        that.updateEntitiesFromTimesliderValue();
                    });
                }
            }
        },

        
        startAddLinkActions: function () {
            var that = this;

            var currentlySelectedEntities = this.getSelectedEntities();

            var selectedNodes = [];
            functional.forIn(currentlySelectedEntities, function (entity, entityIndex) {
                if (entity.type._id === 'NODE') {
                    selectedNodes.push(entity);
                }
            });

            if (selectedNodes.length === 2) {
                var node0 = selectedNodes[0];
                var node1 = selectedNodes[1];
                if (lang.hitch(this, doesLinkAlreadyExist, node0, node1)()) {
                    MessageUtil.showErrorMessage(I18nUtil.get('networkview_alinkalreadyexistsbetweenthosetwonodes'));
                }
                else {
                    var newPrototypeLink = {};
                    lang.mixin(newPrototypeLink, PrototypeEntity.emptyLink);

                    
                    this.model.showAddLinkDialog(this.networkState, node0, node1, newPrototypeLink, function (newLink) {
                        that.networkState.links.push(newLink);

                        that.NetworkStatefulHelper.sync([newLink]);
                        that.NodeLinkFilterHelper.incrementNodeDegrees(newLink);

                        MessageUtil.showSuccessMessage(I18nUtil.get('networkview_linkadded'));
                        lang.hitch(that, updateTimeSlider)();

                        
                        that.UndoStackHelper.clearStackIfContains(that.UndoStackHelper.GROW, [
                            node0.uid,
                            node1.uid
                        ], that);
                    });
                }
            }
            else if (selectedNodes.length > 2) {
                MessageUtil.showErrorMessage(I18nUtil.get('networkview_pleaseselectonlytwonodes'));
            }
            else {
                MessageUtil.showErrorMessage(I18nUtil.get('networkview_youneedtohavetwonodesselected'));
            }
        },

        
        networkCanBeSaved: function () {
            if (((this.networkState.nodes.length < 1) && (this.networkState.groups.length < 1)) || (this.networkState.links.length < 1)) {
                MessageUtil.showErrorMessage(I18nUtil.get('networkview_networkmusthaveatleastonenode'));
                return false;
            }
            if (!this.hasValidLink(this.networkState.links, this.networkState.nodes)) {
                MessageUtil.showErrorMessage(I18nUtil.get('networkview_networkmusthaveatleastonenode'));
                return false;
            }
            return true;
        },

        
        entitiesCanBeRemovedSimpleCheck: function () {
            if (((this.networkState.nodes.length < 1) && (this.networkState.groups.length < 1)) || (this.networkState.links.length < 1)) {
                MessageUtil.showErrorMessage(I18nUtil.get('networkview_networkmusthaveatleastonenode'));
                return false;
            }
            var currentlySelectedEntities = this.getSelectedEntities();
            if (currentlySelectedEntities.length < 1) {
                MessageUtil.showErrorMessage(I18nUtil.get('networkview_selectanentity'));
                return false;
            }
            var selectedNodesAndLinks = getNodesLinksAndGroups(currentlySelectedEntities);
            if (selectedNodesAndLinks.nodes.length === this.networkState.nodes.length) {
                MessageUtil.showErrorMessage(I18nUtil.get('networkview_cannotdeleteentirenetwork'));
                return false;
            }
            var unselectedEntities = this.getUnselectedEntities(selectedNodesAndLinks.groups);
            var unselectedNodesAndLinks = getNodesLinksAndGroups(unselectedEntities);
            var unselectedNodes = this.trimSelectedGroupsFromUnselectedNodes(selectedNodesAndLinks.groups, unselectedNodesAndLinks.nodes);
            if ((unselectedNodes.length < 1) || (unselectedNodesAndLinks.links.length < 1)) {
                MessageUtil.showErrorMessage(I18nUtil.get('networkview_networkmusthaveatleastonenode'));
                return false;
            }
            if (!this.hasValidLink(unselectedNodesAndLinks.links, unselectedNodes)) {
                return false;
            }
            return true;
        },

        
        hasValidLink: function (linkList, nodeList) {
            var link;
            for (var i = 0; i < linkList.length; i++) {
                link = linkList[i];

                if (this.isUnselectedNode(link.fromNodeId, nodeList) && this.isUnselectedNode(link.toNodeId, nodeList)) {
                    return true;
                }
            }
            MessageUtil.showErrorMessage(I18nUtil.get('networkview_networkmusthaveatleastonenode'));
            return false;
        },

        isUnselectedNode: function (id, nodeList) {
            for (var i = 0; i < nodeList.length; i++) {
                if (id == nodeList[i].uid) {
                    return true;
                }
            }
            return false;
        },
        trimSelectedGroupsFromUnselectedNodes: function (groupList, nodeList) {
            var unselectedNodes = [];
            for (var i = 0; i < nodeList.length; i++) {
                if (!this.nodeInSelectedGroup(nodeList[i].uid, groupList)) {
                    unselectedNodes.push(nodeList[i]);
                }
            }
            return unselectedNodes;
        },
        nodeInSelectedGroup: function (id, groupList) {
            for (var i = 0; i < groupList.length; i++) {
                var group = groupList[i];
                for (var j = 0; j < group.children.length; j++) {
                    var child = group.children[j];
                    if (id == child.uid) {
                        return true;
                    }
                }
            }
            return false;
        },


        
        startRemoveEntityActions: function () {
            var that = this;

            var currentlySelectedEntities = this.getSelectedEntities();
            var selectedNodesAndLinks = getNodesAndLinks(currentlySelectedEntities);

            if (!this.entitiesCanBeRemovedSimpleCheck()) {
                return;
            }
            else if (currentlySelectedEntities.length >= 1) {

                var focusedEntityUIDs = [];
                currentlySelectedEntities.forEach(function (entity) {
                    focusedEntityUIDs.push(entity.get('uid'));
                });
                var successMessage = "";
                if (currentlySelectedEntities.length > 1) {
                    successMessage = I18nUtil.get('networkview_entitiesremoved');
                } else {
                    var entityType = currentlySelectedEntities[0].type._id;
                    if (entityType === 'NODE') {
                        successMessage = I18nUtil.get('networkview_noderemoved');
                    } else if (entityType === 'LINK') {
                        successMessage = I18nUtil.get('networkview_linkremoved');
                    }
                    else if (entityType === 'GROUP') {
                        successMessage = I18nUtil.get('networkview_entitiesremoved');
                    }
                }
                currentlySelectedEntities.forEach(function (currentlyFocusedEntity) {
                    var isNode = currentlyFocusedEntity.type._id === 'NODE';
                    var isLink = currentlyFocusedEntity.type._id === 'LINK';
                    var isGroup = currentlyFocusedEntity.type._id === 'GROUP';

                    if (isNode === true) {
                        that.NodeLinkFilterHelper.removeNode(that.networkState, currentlyFocusedEntity.uid);
                        that.UndoStackHelper.clearStackIfContains(that.UndoStackHelper.GROW, [currentlyFocusedEntity.uid], that);
                        that.UndoStackHelper.clearStackIfContains(that.UndoStackHelper.REGROUP, [currentlyFocusedEntity.uid], that);
                    } else if (isLink === true) {
                        that.NodeLinkFilterHelper.removeLink(that.networkState, currentlyFocusedEntity.uid);
                        that.UndoStackHelper.clearStackIfContains(that.UndoStackHelper.GROW, [
                            currentlyFocusedEntity.toNodeId,
                            currentlyFocusedEntity.fromNodeId
                        ], that);
                        that.UndoStackHelper.clearStackIfContains(that.UndoStackHelper.REGROUP, [
                            currentlyFocusedEntity.toNodeId,
                            currentlyFocusedEntity.fromNodeId
                        ], that);
                    }
                    else if (isGroup === true) {
                        that.NodeLinkFilterHelper.removeGroup(that.networkState, currentlyFocusedEntity);
                        that.UndoStackHelper.clearStackIfContains(that.UndoStackHelper.GROW, [currentlyFocusedEntity.uid], that);
                        that.UndoStackHelper.clearStackIfContains(that.UndoStackHelper.REGROUP, [currentlyFocusedEntity.uid], that);
                    }

                });
                that.noEntitiesSelected();
                lang.hitch(that, updateTimeSlider)();
                that.updateEntitiesFromTimesliderValue();

                if (successMessage !== "") {
                    MessageUtil.showSuccessMessage(successMessage);
                    that.markAsUnsavedChanges();
                }

                that.hideNodeSpecificSections();
            }
        },

        
        addGroup: function (groupUid, nodeUidArray) {
            var that = this;

            var arrayContainsGroupUid = function () {
                var containsGroupUid = false;
                for (var i = 0; i < nodeUidArray.length; i++) {
                    var uid = nodeUidArray[i];

                    var entity = lang.hitch(that, getEntityFromUID, uid)();

                    if (entity.type === 'GROUP') {
                        containsGroupUid = true;
                        break;
                    }
                }

                return containsGroupUid;
            };

            if (nodeUidArray.length < 2) {
                MessageUtil.showErrorMessage(I18nUtil.get('networkview_theremustbetwoormorenodesselected'));
            }
            else if (arrayContainsGroupUid() === true) {
                MessageUtil.showErrorMessage(I18nUtil.get('networkview_groupscannotbenested'));
            }
            else {
                var jsonGroupArray = [];
                if (!groupUid) {
                    groupUid = GUIDUtil.generate();
                }
                functional.forIn(nodeUidArray, function (nodeUid, nodeUidIndex) {
                    var proposedEntity = lang.hitch(that, getEntityFromUID, nodeUid)();
                    if (proposedEntity && proposedEntity.type === 'NODE') {
                        var newJsonGroup = {};
                        lang.mixin(newJsonGroup, PrototypeEntity.emptyGroup);

                        newJsonGroup.uid = groupUid;
                        newJsonGroup.nodeUid = nodeUid;

                        that.networkState.groups.push(newJsonGroup);

                        jsonGroupArray.push(newJsonGroup);
                    }
                });

                that.NetworkStatefulHelper.addGroup(groupUid, jsonGroupArray, that);
                that.markAsUnsavedChanges();
            }
        },

        removeGroup: function (groupUid) {
            var that = this;
            functional.forIn(this.networkState.groups, function (group, groupIndex) {
                if (group.uid === groupUid) {
                    delete that.networkState.groups[groupIndex];
                }
            });

            this.NetworkStatefulHelper.removeGroup(groupUid, this);
            that.markAsUnsavedChanges();
        },

        hideNodeSpecificSections: function () {
            this.networkDetailsController.hide();
        },

        
        cleanup: function () {
            if (model) {
                model.cleanupView();
            }
        },

        
        setFocusedEntityId: function (entityId) {
            this.networkState.focusedEntityId = entityId;
        },

        
        getFocusedEntityId: function (entityId) {
            return this.networkState.focusedEntityId;
        },

        
        showPropertiesSection: function (previousEntityId, doubleClicked) {
            var that = this;
            var currentlyFocusedEntity = lang.hitch(this, getCurrentlyFocusedEntity)();

            if (currentlyFocusedEntity) {
                
                if (previousEntityId !== this.networkState.focusedEntityId) {
                    this.networkPropertiesController.showEntityProperties(currentlyFocusedEntity, this.networkState, this.networkMapController.isOpen(), this.supportLinkDetails);
                    var selectedUIDs = this.NLDHelper.getSelectedUids();
                    if (selectedUIDs.length > 1) {
                        this.networkPropertiesController.multipleEntitiesSelected();
                    } else if (selectedUIDs.length < 1) {
                        this.networkPropertiesController.noEntitiesSelected();
                    }
                }

                
                if (currentlyFocusedEntity.type === 'GROUP') {
                    this.networkDetailsController.hide();
                }
                
                else if (currentlyFocusedEntity.type === 'LINK' && !this.supportLinkDetails) {
                    this.networkDetailsController.hide();
                }
                
                else if ((this.networkDetailsController.isOpen() || doubleClicked) && ((currentlyFocusedEntity.type === 'NODE') || ((currentlyFocusedEntity.type === 'LINK') && this.supportLinkDetails))
                    && (previousEntityId !== this.networkState.focusedEntityId)) {
                    
                    var loadingComplete = false;
                    var completionFunction = function () {
                        return loadingComplete;
                    };
                    this.networkPropertiesController.startLoadingSpinner(completionFunction);
                    this.networkDetailsController.startLoadingSpinner(completionFunction);

                    this.showDetails(currentlyFocusedEntity, function () {
                        loadingComplete = true;
                    }, function () {
                        loadingComplete = true; 
                        that.networkDetailsController.showLoadingError(true, currentlyFocusedEntity.type);
                    });
                }
            }
        },

        multipleEntitiesSelected: function () {
            this.networkPropertiesController.multipleEntitiesSelected();
        },

        noEntitiesSelected: function () {
            this.networkPropertiesController.noEntitiesSelected();
        },

        
        isPropertiesSectionOpen: function () {
            return this.networkPropertiesController.isOpen();
        },

        
        startHighlightNetworkActions: function (facets) {
            var facetsToUse = facets || this.networkFacetController.facets;

            var that = this;
            this.CustomMeasuresHelper.performAspectTracing(this.networkState, facetsToUse, function (entity, highlight) {
                that.aspectTracingAction(entity, highlight);
            });
            that.model.refreshView();
        },

        
        showDetails: function (entity, callback, errorCallback) {
            var that = this;

            var currentlySelectedEntities = this.getSelectedEntities();

            if (currentlySelectedEntities.length === 1 && ((currentlySelectedEntities[0].type._id === 'NODE') || (currentlySelectedEntities[0].type._id === 'LINK'))) {
                this.networkDetailsController.show(this.vertical, currentlySelectedEntities[0].uid, entity, function () {
                    
                    if (callback) {
                        callback();
                    }
                }, errorCallback);
            } else {
                MessageUtil.showErrorMessage(I18nUtil.get('networkview_youneedtohaveanodeselected'));

                if (callback) {
                    callback();
                }
            }
        },

        
        filterDataFromTimeSliderValue: function (timeSliderValue) {
            var timesliderAlgorithm = this.model.getTimeSliderAlgorithm();

            var that = this;
            this.TimeSliderHelper.filterDataBasedOnTimeSliderValue(this.networkState, timeSliderValue, timesliderAlgorithm, {
                nodeChangeDateAction: function () {
                    that.updateEntitiesFromTimesliderValue();
                },
                linkDimAction: function (link, dim) {
                    that.timeSliderLinkDimAction(link, dim);
                },
                nodeDimAction: function (node, dim) {
                    that.timeSliderNodeDimAction(node, dim);
                }
            });
        },

        
        markAsUnsavedChanges: function () {
            if ((typeof this.snaSaveButton != 'undefined') && (this.snaSaveButton.get('disabled') === false)) {
                ApplicationContextModel.set('unsavedChanges', true);
            }
        },

        
        markChangesAsSaved: function () {
            if ((typeof this.snaSaveButton != 'undefined') && (this.snaSaveButton.get('disabled') === false)) {
                ApplicationContextModel.set('unsavedChanges', false);
            }
        },


        
        saveNetwork: function () {
            var that = this;
            var fieldsToSave = [
                'uid',
                'positionX',
                'positionY',
                'nodeLabel',
                'nodeType',
                'evolutions2',
                'nodeToolTip',
                "isNodeHidden",
                "nodeDegrees",
                "borderSize",
                "type",
                "evolves",
                'latitude',
                'longitude',
                "linkDirection"
            ];

            if (!this.networkCanBeSaved()) {
                return;
            }

            this.NetworkStatefulHelper.reverseSync(this.networkState.nodes);
            this.NetworkStatefulHelper.saveTimeReverseSync(this.networkState.nodes);

            var subsetNodes = JSONUtil.getSubsetObjectArray(this.networkState.nodes, fieldsToSave);

            var customMeasures = {};
            if (this.networkState.tables) {
                customMeasures = this.networkState.tables.MEASURES;
            }

            var modifiedLinks = lang.clone(this.networkState.links);
            functional.forIn(modifiedLinks, function (link) {
                
                if (link.linkDirection && link.linkDirection === 'TO') {
                    link.type = 'VECTOR';
                }
                else {
                    link.type = 'LINK';
                }

                if (link.linkStartDt) {
                    if (link.linkStartDt.getTime) {
                        link.linkStartDt = link.linkStartDt.getTime();
                    }
                    else {
                        link.linkStartDt = new Date(link.linkStartDt).getTime();
                    }
                }

                if (link.linkEndDt) {
                    if (link.linkEndDt.getTime) {
                        link.linkEndDt = link.linkEndDt.getTime();
                    }
                    else {
                        link.linkEndDt = new Date(link.linkEndDt).getTime();
                    }
                }
            });

            networkData.saveNetwork(
                this.networkState.actionableEntityId,
                this.networkState.actionableEntityType,
                subsetNodes,
                modifiedLinks,
                this.networkState.groups,
                this.networkState.vertical,
                customMeasures,
                function (data) {
                    if (data.success === 'true' || data.success === true) {
                        MessageUtil.showSuccessMessage(I18nUtil.get('networkview_networksaved'));
                        that.markChangesAsSaved();
                    }
                    else {
                        MessageUtil.showErrorMessage(I18nUtil.get('networkview_saveunsuccessfulpleaseseeadministrator'));
                    }
                }
            );
        },

        
        showNetworkInfoButtonClicked: function () {
            var that = this;
            require([
                'sna/applicationComponents/NetworkView/widgets/NetworkInfoDump/NetworkInfoDump/NetworkInfoDump'
            ], function (NetworkInfoDump) {
                var nid = new NetworkInfoDump();
                nid.print(that.networkState, that.NetworkStatefulHelper.getStatefulEntityArray());
            });
        },

        
        attachAnnotation: function (toolTip) {
            var currentlyFocusedEntity = lang.hitch(this, getCurrentlyFocusedEntity)();

            if (currentlyFocusedEntity.type === 'NODE') {
                currentlyFocusedEntity.nodeToolTip = toolTip;
                MessageUtil.showSuccessMessage(I18nUtil.get('networkview_annotationattachedtonode'));
            }
            else if (currentlyFocusedEntity.type === 'LINK') {
                currentlyFocusedEntity.linkLabel = toolTip;
                MessageUtil.showSuccessMessage(I18nUtil.get('networkview_annotationattachedtolink'));
            }
        },

        
        attachAnnotationToNode: function (uid, toolTip) {
            var entity = lang.hitch(this, getEntityFromUID, uid)();

            if (entity.type === 'NODE') {
                entity.nodeToolTip = toolTip;
                MessageUtil.showSuccessMessage(I18nUtil.get('networkview_annotationattachedtonode'));
            }
        },

        getAnnotation: function (uid) {
            var annotation = null;

            var entity = lang.hitch(this, getEntityFromUID, uid)();
            if (entity) {
                if (entity.type === 'NODE') {
                    annotation = entity.nodeToolTip;
                }
                else if (entity.type === 'LINK') {
                    annotation = entity.linkLabel;
                }
            }

            return annotation;
        },

        
        timeSliderNodeChangeDateAction: function (node, evolutionsIndex) {
            var getLatestChangeDate = function (node, changeDate) {
                var latest = "0";

                functional.forIn(node.evolutions2, function (evolution, evolutionIndex) {
                    var currentEvolution = Number(evolutionIndex);
                    var nChangeDate = Number(changeDate);

                    if (currentEvolution <= nChangeDate) {
                        latest = ('' + currentEvolution);
                    }
                });

                return latest;
            };

            var latestChangeDate = getLatestChangeDate(node, evolutionsIndex);
            this.NodeLinkFilterHelper.updateNode(node, node.evolutions2[latestChangeDate]);
            this.updateEntity(node.uid, node, 'useReverseSync');
        },

        
        updateEntitiesFromTimesliderValue: function () {
            var that = this;
            var timesliderValue = this.model.getTimeSliderValue();
            functional.forIn(this.networkState.nodes, function (node, nodeIndex) {
                that.timeSliderNodeChangeDateAction(node, timesliderValue);
            });
        },

        
        timeSliderLinkDimAction: function (link, dim) {
            var statefulArrayIndex = this.NetworkStatefulHelper.getArrayIndex(link.uid);
            
            this.NLDHelper.dimEntity(statefulArrayIndex, dim);
        },

        
        timeSliderNodeDimAction: function (node, dim) {
            var statefulArrayIndex = this.NetworkStatefulHelper.getArrayIndex(node.uid);
            this.NLDHelper.dimEntity(statefulArrayIndex, !dim);
        },

        
        aspectTracingAction: function (entity, highlight) {
            this.updateEntity(entity.uid, {
                highlight: highlight
            }, 'useReverseSync');
        },

        
        startRunLayoutAlgorithmActions: function (algorithmString, params, vertical, completionCallback) {
            var that = this;
            var subsetNodes = JSONUtil.getSubsetObjectArray(this.networkState.nodes, ['uid', 'positionX', 'positionY']);
            var subsetLinks = JSONUtil.getSubsetObjectArray(this.networkState.links, ['uid', 'fromNodeId', 'toNodeId']);
            networkData.getLayout(subsetNodes, subsetLinks, vertical, algorithmString, params, function (nodes) {
                functional.forIn(nodes, function (node, nodeIndex) {
                    that.updateEntity(node.uid, {
                        positionX: node.positionX,
                        positionY: node.positionY
                    });
                });

                that.NLDHelper.center();

                if (completionCallback) {
                    var isSuccessful = true;
                    if (nodes === "" || nodes === null || nodes === "null") { 
                        dialog.showDialog();
                        isSuccessful = false;
                    }
                    completionCallback(isSuccessful);
                }
            });
        },

        
        startGrowNetworkActions: function (nodeUid, nodeType) {
            var that = this;

            
            var loadingComplete = false;
            var completionFunction = function () {
                return loadingComplete;
            };

            new LoadingSpinner().target(that.snamaincontentpane).until(completionFunction).startSpinners(function () {
            });

            
            this.NetworkStatefulHelper.reverseSync(this.networkState.nodes);

            var subsetNodes = JSONUtil.getSubsetObjectArray(this.networkState.nodes, ['uid', 'positionX', 'positionY']);
            var subsetLinks = JSONUtil.getSubsetObjectArray(this.networkState.links, ['uid', 'fromNodeId', 'toNodeId']);

            
            
            
            networkData.growSocialNetwork(this.vertical, this.networkState.actionableEntityId, this.networkState.actionableEntityType, nodeUid, nodeType, subsetNodes, subsetLinks, function (data) {
                
                var originalNodes = that.networkState.nodes.slice(0);
                var originalLinks = that.networkState.links.slice(0);

                var originalNodesMap = that.convertArrayToMap(originalNodes);
                var originalLinksMap = that.convertArrayToMap(originalLinks);
                var nodeMap = that.convertArrayToMap(data.nodes);
                var linkMap = that.convertArrayToMap(data.links);

                
                if (data.tables) {
                    that.networkState.tables = data.tables;
                }

                
                for (var key in nodeMap) {
                    var node = nodeMap[key];
                    var existingNode = originalNodesMap[key];

                    if (!existingNode) {
                        
                        that.networkState.nodes.push(node);
                        originalNodesMap[key] = node;
                    } else if (!Object.keys(node.evolutions2).length === 0) {
                        lang.mixin(existingNode, node);
                    }

                    
                    that.updateEntity(node.uid, {
                        positionX: node.positionX,
                        positionY: node.positionY
                    });
                }

                
                for (var key in linkMap) {
                    var link = linkMap[key];
                    var existingLink = originalLinksMap[key];

                    if (existingLink) {
                        lang.mixin(existingLink, link);
                    } else {
                        that.networkState.links.push(link);
                        
                        
                    }
                }

                if (nodeMap) {
                    var justNodesAndLinks = data.nodes.concat(data.links);
                    var originalNodesAndLinks = originalNodes.concat(originalLinks);

                    var justNodesAndLinksMap = that.convertArrayToMap(justNodesAndLinks);
                    var originalNodesAndLinksMap = that.convertArrayToMap(originalNodesAndLinks);

                    var getNewNodesAndLinks = function (originalEntityMap, newEntityMap) {
                        var differenceEntityArray = [];

                        var hasEntity = function (map, uid) {
                            return !!map[uid]; 
                        };

                        for (key in newEntityMap) {
                            if (!hasEntity(originalEntityMap, key)) {
                                differenceEntityArray.push(newEntityMap[key]);
                            }
                        }

                        return differenceEntityArray;
                    };

                    var newNodesAndLinks = getNewNodesAndLinks(originalNodesAndLinksMap, justNodesAndLinksMap);

                    that.NetworkStatefulHelper.sync(justNodesAndLinks);

                    
                    if (data.facets) {
                        that.networkState.facets = data.facets;
                    }
                    that.networkFacetController.refresh(that.networkState.facets);

                    

                    
                    var timeSliderValues = that.TimeSliderHelper.getTimeSliderValues(that.networkState);
                    that.model.setupTimeSlider(timeSliderValues);
                    that.updateEntitiesFromTimesliderValue();
                    that.filterDataFromTimeSliderValue(that.model.getTimeSliderValue());

                    var newNodesAndLinksUids = [];
                    functional.forIn(newNodesAndLinks, function (newEntity) {
                        newNodesAndLinksUids.push(newEntity.uid);
                    });

                    that.snaCollapseExpandedNodesButton.set('disabled', false);
                    that.UndoStackHelper.push(that.UndoStackHelper.GROW, {
                        keyUid: nodeUid,
                        uids: newNodesAndLinksUids
                    });

                    
                    
                    
                    
                    
                    
                    var sourceNode = lang.hitch(that, getEntityFromUID, nodeUid)();
                    that.NetworkStatefulHelper.sync([sourceNode]);

                    
                    that.networkMapController.updateMenuState();
                    that.markAsUnsavedChanges();
                }
                else {
                    console.warn("data.nodes is undefined")
                }
                loadingComplete = true; 
            });
        },

        destroy: function () {
            this.networkFacetController.destroyFacets();

            this.inherited(arguments);
        },

        collapseExpandedNodes: function () {
            var that = this;
            that.markAsUnsavedChanges();
            this.CollapseExpandedNodesHelper.perform(this).then(function (networkState) {
                
                lang.hitch(that, updateTimeSlider)();
                that.updateEntitiesFromTimesliderValue();
                that.filterDataFromTimeSliderValue(that.model.getTimeSliderValue());
            });
        },

        getGlobalSearchQueryFunctions: function () {
            return this.networkFacetController.getGlobalSearchQueryFunctions();
        },

        refreshNetworkDetailsGrids: function () {
            this.networkDetailsController.refreshDataGrids();
        },

        
        reloadNetwork: function (source) {

            
            if (ApplicationContextModel.get('unsavedChanges')) {
                var confirm = window.confirm(I18nUtil.get('global_unsavedchanges'));
                if (confirm === false) {
                    return;
                }
            }

            
            query(".networkdiagram").forEach(domConstruct.destroy);
            var that = this;
            that.loading.set('loading', true);

            
            that.networkState.nodes = {};
            that.networkState.links = {};
            that.networkState.groups = [];
            that.networkState.tables = {};
            that.networkState.facets = {};
            that.NetworkStatefulHelper.clear();

            that.UndoStackHelper.clearStack(that.UndoStackHelper.GROW, that);
            that.UndoStackHelper.clearStack(that.UndoStackHelper.REGROUP, that);

            var vertical = that.networkState.vertical;

            
            var loadingComplete = false;
            var completionFunction = function () {
                return loadingComplete;
            };

            new LoadingSpinner().target(that.snamaincontentpane).until(completionFunction).startSpinners(function () {
            });


            
            
            
            
            
            
            
            
            
            
            
            
            
            
            
            
            
            
            
            
            
            
            
            

         if(source == "NETWORK_SOURCE_DATABASE") {
                networkData.checkForSavedNetwork(vertical, that.networkState.entity.actionableEntityId, that.networkState.entity.actionableEntityType, function (savedNetworkExists) {
                        if(savedNetworkExists === "true") {
                            
                            getNetworkAndFacetsFromServer("NETWORK_SOURCE_STOREDPROCESS", function (stpData) {
                                var colorModel = stpData.colorModel;

                                
                                getNetworkAndFacetsFromServer("NETWORK_SOURCE_DATABASE", function (results) {
                                    results.colorModel = colorModel;
                                    processNetworkAndFacetsFromServer(results);
                                });
                            });
                        }else {
                            getNetworkAndFacetsFromServer("NETWORK_SOURCE_STOREDPROCESS", processNetworkAndFacetsFromServer);
                        }
                });
            }
            else{
                getNetworkAndFacetsFromServer("NETWORK_SOURCE_STOREDPROCESS", processNetworkAndFacetsFromServer);
            }

            function getNetworkAndFacetsFromServer(preferredSource, callback){
                networkData.getSocialNetworkAndFacets(vertical, that.networkState.entity.actionableEntityId, that.networkState.entity.actionableEntityType, preferredSource, callback);
            }

            function processNetworkAndFacetsFromServer(results){
                UserData.getUser(function (userData) {
                    loadingComplete = true;

                    
                    that.networkPropertiesController.hide();
                    that.networkDetailsController.hide();

                    lang.mixin(that.networkState, results);

                    lang.hitch(that, isSaveNetworkAllowedAction, results.isSaveNetworkAllowed)();

                    var justNodesAndLinks = results.nodes;
                    justNodesAndLinks = justNodesAndLinks.concat(results.links);
                    if (results.groups != null) {
                        results.groups = removeSingleGroups(results.groups);
                        justNodesAndLinks = justNodesAndLinks.concat(results.groups);
                    }
                    that.NetworkStatefulHelper.sync(justNodesAndLinks);

                    that.model.resetLayoutLabel(that.snaRunLayoutAlgorithmDropdownButton_fr1);
                    that.model.render(that.NetworkStatefulHelper.getStatefulEntityArray(), results.plugins, vertical, function () {
                        
                        var timeSliderValues = that.TimeSliderHelper.getTimeSliderValues(that.networkState);
                        that.model.setupTimeSlider(timeSliderValues);
                        that.model.resetTimeSlider(timeSliderValues);

                        
                        PluginUtil.createPluginsFromMap(results.plugins, {
                            component: that,
                            networkState: that.networkState,
                            data: results
                        }, vertical);

                        that.updateEntitiesFromTimesliderValue();
                        that.filterDataFromTimeSliderValue(that.model.getTimeSliderValue());

                        that.NetworkStatefulHelper.handleRawGroups(results);

                        
                        that.networkFacetController.destroyFacets();
                        that.networkFacetController.start(that.networkState.facets);
                        that.networkFacetController.refresh(that.networkState.facets);

                        
                        that.snaRegroupNodesButton.set('disabled', 'disabled');
                        that.snaCollapseExpandedNodesButton.set('disabled', 'disabled');

                        
                        that.networkMapController.updateMenuState();
                        that.loading.set('loading', false);
                    });
                });
            }
        },

        getEntityFromUID: function (uid) {
            var entity = null;
            if (uid) {
                entity = lang.hitch(this, getEntityFromUID, uid)();
            }

            return entity;
        },

        convertArrayToMap: function (l) {
            return l.reduce(function (o, v, i) {
                o[v.uid] = v;
                return o;
            }, {});
        }
    });
});

               