
                   
define([
    'dojo/_base/array',
    'dojo/query',
    'sna/data/AlertDetailsData',
    'sna/util/MessageUtil',
    'sna/util/I18nUtil',
    'sna/util/CustomView',
    'sna/util/DebugUtil',
    'dojo/Deferred',
    'dojox/lang/functional',
    'dijit/registry',
    'dojo/_base/declare',
    'sna/model/Locator',
    'sna/util/PluginUtil'
], function( array, query, AlertDetailsData, MessageUtil, I18nUtil, CustomView, DebugUtil, Deferred, functional, registry, declare, locator, PluginUtil ) {
    "use strict";

    return declare([], {
        
        constructor: function( args ) {
            this.dispositionController = args.dispositionController;
            this.maximizedChartsController = args.maximizedChartsController;
            this.alertDetailsFacetController = args.alertDetailsFacetController;
            this.customView = new CustomView({
                snaView: "DetailsView",
                menuButton: registry.byId('detailscustomviewbutton'),
                dropDownMenu: registry.byId('detailscustomviewdropdownmenu')
            });
            
            
            this.okToRecallExport = false;
        },

        
        listHasBeenExported: function(){
            this.okToRecallExport = true;
        },

        
        _runHiddenPlugins: function(pluginDataMap, plugins) {
        	PluginUtil.createPluginsFromMap(plugins, pluginDataMap);
        },

        _getView: function(entity, vertical) {
            var deferred = new Deferred({});

            var that = this;
            var view = 'sna/views/StandardDetailsView/StandardDetailsView';

            
            
            
            
            var customDetailView = DebugUtil.customDetailsView();
            if(typeof(customDetailView) !== 'undefined') {
                view = customDetailView;
            }

            var importView = function() {
                require([
                    view
                ], function(View) {
                    deferred.resolve(new View({
                        dispositionController: that.dispositionController,
                        maximizedChartsController: that.maximizedChartsController,
                        alertDetailsFacetController: that.alertDetailsFacetController,
                        customView: that.customView,
                        title: ''
                    }));
                });
            };

            AlertDetailsData.getDetailsView(vertical, entity).then(function(data) {
                if(data && data.length > 0) {
                    view = data[0].module;
                }

                importView();
            }, function(err) {
                importView();
            });

            return deferred;
        },
        getEntity: function() {
            return this.model.entity;
        },

        
        start: function (entity, vertical) {
            var that = this;

            this.customView.buildCustomViewList(vertical,this);

            this._getView(entity, vertical).then(function(view) {
                that.model = view;
                registry.byNode(locator.locate().maincontent()).addChild(view);
                view.start(entity, vertical).then(function(){
                    that.customView.show();
                    that.customView.applyLastKnownView();
                    registry.byId('detailscustomviewbutton').set('disabled', false);
                    if(!that.isNetworkViewLoaded()) registry.byId('detailscustomviewbutton').focus();
                });
            });
        },


        
        isLoaded:function () {
            var isLoaded = false;

            if (this.model) {
                isLoaded = this.model.isLoaded();
            }

            return isLoaded;
        },

        isNetworkViewLoaded: function() {
            var isLoaded = false;

            if (this.model) {
                isLoaded = this.model.isNetworkViewLoaded();
            }

            return isLoaded;
        },

        printNetworkView: function() {
            if(this.model) {
                this.model.printNetworkView();
            }
        },

        
        getPrintData: function() {
            return this.model.getPrintData();
        },

        
        getPrintableTables: function() {

            return this.model.getPrintData();
        },

        cleanup: function() {
            this.alertDetailsFacetController.destroy();
            this.customView.hide();

            if(this.model && this.model.destroyRecursive) {
                this.model.destroyRecursive();
            }
        },

        
        updateDispositionItemsList: function(entitiesArray, view) {
            this.model.updateDispositionItemsList(entitiesArray, view);
        },

        
        startNetworkViewController: function( vertical, actionableEntity ) {
            this.model.startNetworkViewController(vertical, actionableEntity);
        },

        showMaximizedCharts: function() {
        	this.model.showMaximizedCharts();
        },

        refreshDataGrid: function() {
            this.okToRecallExport = false;
            this.model.refreshDataGrid();
        },

        
        
        getFilterValues: function() {
            return null;
        },

        
        
        setFilterValues: function(filterValues ) {
        },

        setGridPreferences: function(preferences ) {
            var grids = this.model._alertTransactionDataGrids;
            var preferencesFound = false;

            
            functional.forIn(this.model._printCharts, function (chart, chartName) {
                if("tableGroup" === chart.chartType){
                    var tables = chart.rows;
                    for (i = 0; i < tables.length; i++) {
                        tables[i].d_grid.setGridPreferences(preferences[tables[i].label]);
                    }
                }
            });

            for(var i=0;i< grids.length;i++){
                var numMatchingPreferences = grids[i].setGridPreferences(preferences[grids[i].name]);
                if(grids[i].name != "Related_Alerts" && numMatchingPreferences > 0){
                    preferencesFound = true;
                }
            }
            if(!preferencesFound) MessageUtil.showErrorMessage(I18nUtil.get("custom_views_wrong_alert_type"));
        },

        gridPreferences: function() {
            var grids = this.model._alertTransactionDataGrids;
            var that = this;
            var gridPreferences = {};
            
            for(var i=0;i< grids.length;i++){
                gridPreferences[grids[i].name] = grids[i].gridPreferences();
            }
            
            functional.forIn(this.model._printCharts, function (chart, chartName) {
                var tables = chart.rows;
                if(tables){
                    for (i = 0; i < tables.length; i++) {
                        if("tableGroup" === chart.chartType){
                            gridPreferences[tables[i].label] = tables[i].d_grid.gridPreferences();
                        }
                    }
                }
            });
            return gridPreferences;
        },


        getFilterQueryFunction: function () {
            return this.alertDetailsFacetController.getFilterQueryFunction();
        },

        keepRow: function (rowData) {
            return this.alertDetailsFacetController.keepRow(rowData);
        },

        isDGridReady: function() {
            return this.model.isDGridReady();
        }
    }); 
});
               