
                   
define([
    'dijit/registry',
    'fraudCommons/widgets/LoadingSpinner/LoadingSpinner',
    '../util/CustomView',
    '../util/ErrorUtil',
	'../util/ContextPreferenceUtil',
	'../util/PluginUtil',
    'sna/util/OverlayUtil',
    'dojo/_base/declare',
    'sna/model/Locator',
    'sna/data/AlertsViewData',
    'sna/model/AlertsViewModel'
], function(registry, LoadingSpinner, CustomView, ErrorUtil, ContextPreferenceUtil, PluginUtil, overlayUtil, declare, locator, alertsViewData, AlertsViewModel ) {
	return declare([], {
		constructor: function( args ) {
            this.viewSwitcherController = args.viewSwitcherController;
			this.alertsListFacetController = args.alertsListFacetController;
			this.alertDetailsController = args.alertDetailsController; 
			this.dispositionController = args.dispositionController; 
			this.externalAppController = args.externalAppController;
            this.customView = new CustomView({
                snaView: "alertsViewFacets",
                menuButton: registry.byId('alertscustomviewbutton'),
                dropDownMenu: registry.byId('alertscustomviewdropdownmenu')
            });
            
            
            this.okToRecallExport = false;
        },
		
		
		start: function(vertical, submissions, fetchEntities, useCached ) {
			
            if(fetchEntities) {
                this.model && this.model.destroyRecursive(true);
                delete this.model;
                var globalSearch = this.alertsListFacetController.facets.global_search_facet;
                if(globalSearch) {
                    globalSearch.destroyRecursive();
                }
            }
			if(!this.model ) {
                this.model = new AlertsViewModel({
                    controller: this,
                    d_context: locator.locate().maincontent()
                });
                this.alertsListFacetController.start(locator.locate().alertsfacetcontent(), locator.locate().globalsearch());
                this.fetchAlertedEntities({}, vertical, submissions, true, useCached);
            }
            else if (fetchEntities) {
                this.fetchAlertedEntities({}, vertical, submissions, fetchEntities, useCached);
            }
            this.customView.buildCustomViewList(vertical,this);
		},

        
        listHasBeenExported: function(){
            this.okToRecallExport = true;
        },

		
		alertClicked: function(entity, vertical, submissions ) {
			this.viewSwitcherController.switchToDetailsPage(entity, vertical, submissions);
		},
		
		cleanup: function(refreshing) {
			if(this.model) {
			    this.model.cleanupView( refreshing );
			}

            var globalSearch = this.alertsListFacetController.facets.global_search_facet;
            if (globalSearch) {
                if (!refreshing) {
                    globalSearch.hide();
                }
            }
		},

        destroyView: function(refreshing) {
            if(this.model) {
                this.model.destroyView(refreshing);
            }
        },

        hideCustomViewMenu: function(){
            this.customView.hide();
        },

        
        setFacetHelperCustomViewController: function(){
            this.alertsListFacetController.setFacetHelperCustomViewController(this);
        },

        unhide: function() {
            if(this.model) {
                this.model.unhideView();
            }
            this.customView.show();

            var globalSearch = this.alertsListFacetController.facets.global_search_facet;
            if (globalSearch) {
                globalSearch.show();
            }
        },

		
		getPrintData: function() {
			return this.model.getPrintData();
		},

		dispositionCompleted: function() {
            this.model.dispositionCompleted();
		},

		
		loadPlugins: function(plugins, data, vertical) {
            if(typeof(plugins) === 'undefined' || typeof(data) === 'undefined' || typeof(vertical) === 'undefined') {
                console.warn('AlertsViewController.loadPlugins: a parameter was undefined', plugins, data, vertical);
            }

            PluginUtil.createPluginInstances(plugins, {alertsData: data}, vertical);
		},

        
        getSelectedAlerts: function() {
            if(this.model) {
                return this.model.getSelectedAlerts();
            }
            else {
                return {};
            }
        },
        
        getHeaderLabels: function() {
            return this.model.getHeaderLabels();
        },
        
        isHiddenColumn: function(column) {
            return this.model.isHiddenColumn(column);
        },

		getAdvancedSearchParameters: function() {
			if(this.model) {
				return this.model.getAdvancedSearchParameters();
			}
			else {
				return {};
			}
		},
		
		refreshDataGrid: function() {
            this.alertsListFacetController.setClearCurrentFilterButtonVisibility();
            this.model.refreshDataGrid();
            this.okToRecallExport = false;
		},
		
		getColumnIndexFromColumnIdentifier: function( columnIdentifier ) {
			return this.model.getColumnIndexFromColumnIdentifier(columnIdentifier);
		},
		
		isDGridReady: function() {
			return this.model.isDGridReady();
		},

        
        removeCheckedRows: function() {
            this.model.removeCheckedRows();
            this.model.refreshDataGrid();
        },

        
        removeRows: function(entityIds) {
            this.model.removeRows(entityIds);
            this.model.refreshDataGrid();
        },

        
        fetchAlertedEntities:function ($context, vertical, submissions, fetchEntities, useCached) {
            var that = this;
            
            if(fetchEntities || typeof(fetchEntities) === 'undefined') {
                var completedLoadingFlag = false;
                if(!useCached) {
                    new LoadingSpinner().target(registry.byId("container")).until(function() { return completedLoadingFlag; }).startSpinners(function() {});
                }

	            alertsViewData.getAlertedEntities(vertical, submissions, 'en_US', function (data) {
                    
                    that.model.destroyDataGrid();

                    if(that.model.isDGridReady()) {
                        console.warn('AlertsViewController: AlertsViewModel did not have d_datagrid reset to undefined, probably due to improper cleanup');
                    }

	            	that.alertsListFacetController.addFacets(data.facets);

                    if(data === ""){
                        data = {error:{code:"error_no_data"}};
                    }
                    var errorOccurred = typeof(data.error) !== 'undefined';
                    if(errorOccurred) {
                        console.warn('AlertsViewController.fetchAlertedEntities: alerted entities had no data ', data);
                        data = ErrorUtil.createErrorTable(data.error, 'MEASURES', {facets:[]});
                    }
                    
                    var showFacetPanel = ContextPreferenceUtil.getPreference(ContextPreferenceUtil.SHOW_ALERTS_FACET_PANEL);
                    if(typeof(showFacetPanel) === 'undefined') {
                        showFacetPanel = 'y';
                    }

                	that.model.render(vertical, data, submissions, showFacetPanel);
                    that.loadPlugins(data.plugins, data, vertical);
	                overlayUtil.hideOverlay();

	                completedLoadingFlag = true;
	            }, useCached);
            }
            else {
            	console.warn('AlertsViewController.fetchAlertedEntities: not fetching any data');
            	
            	that.alertsListFacetController.addFacets({});
            	that.model.render(vertical, {}, submissions);
                overlayUtil.hideOverlay();
            }
            
        }, 

        getFilterValues: function() {
            return this.alertsListFacetController.getFilterValues();
        },

        setFilterValues: function(filterValues ) {
            this.alertsListFacetController.setFilterValues(filterValues);
        },

        setGridPreferences: function(preferences ) {
            this.model._d_dataGrid.setGridPreferences(preferences);
        },

        gridPreferences: function() {
            return this.model._d_dataGrid.gridPreferences();
        },

        getFilterQueryFunction:function () {
            return this.alertsListFacetController.getFilterQueryFunction();
        },

        keepRow:function (rowData) {
            return this.alertsListFacetController.keepRow(rowData);
        },

        updateDispositionItemsList:function (entities, view) {
            this.dispositionController.updateDispositionItemsList(entities, 'alerts');
        },

        refreshDispositionItemsList: function() {
            this.updateDispositionItemsList(this.model.getCheckedEntities(), 'alerts');
        },


        startDispositionController:function (vertical, callback) {
            this.dispositionController.start(vertical, callback, false);
        },
        
        updateExternalAppsList:function(entities, view) {
        	this.externalAppController.updateExternalAppsList(entities, 'alerts');
        },

        startExternalAppController:function (vertical, callback) {
            this.externalAppController.start(vertical, callback);
        }
	}); 
}); 

               