
                   
define([
    'dijit/registry',
    'dojo/_base/declare',
    'dojo/request',
    'dojo/request/iframe',
    'sna/model/ExportDialog',
    'sna/util/MessageUtil',
    'sna/util/I18nUtil',
    'dojo/_base/array',
    'dojo/_base/lang',
    'sna/model/ContextModel'
], function (registry, declare, request, iframe, ExportDialog, MessageUtil, I18nUtil, array, lang, ContextModel) {
    
    
    
    var maxExportCharacterSize = 1000000;
    var viewToDgridColumnMap;

    return declare([], {
        dialog: null,
        constructor: function (args) {
            this.alertsViewController = args.alertsViewController;
            this.alertDetailsController = args.alertDetailsController;
            this.lastExport = {};
        },

        exportData: function(){
            var snaSheet = this.buildSNASheet();
            var that = this;
            this.dialog = new ExportDialog();
            if(("alerts" === snaSheet.whichView)&&(snaSheet.elementsAreChecked)){
                this.lastExport = {};
                this.initiateDownload(JSON.stringify(snaSheet));
                that.lastExport.fileName = snaSheet.fileName;
                that.lastExport.excludedItemsList = [];
                that.dialog.cleanupView();
            }else{
                this.checkForOkToREcallExport(snaSheet.whichView);
                this.dialog.show(snaSheet, this.initiateDownload, this.lastExport);
            }
            this.signalExportComplete(snaSheet.whichView);
        },

        checkForOkToREcallExport: function(view){
            
            

            if((view === "alerts") && !this.alertsViewController.okToRecallExport) {
                this.lastExport.view = "";
            }else if((view !== "alerts") && !this.alertDetailsController.okToRecallExport){
                this.lastExport.view = "";
            }

        },

        signalExportComplete: function(view){
            if(view === "alerts"){
                this.alertsViewController.listHasBeenExported();
            }else {
                this.alertDetailsController.listHasBeenExported();
            }
        },


        findIndexByColumnId: function(dgridColumnList, customViewColumnId){
            var index;
            var dgridColumn;
            var columnId;


            for (var i = 0; i < dgridColumnList.length; ++i) {
                dgridColumn = dgridColumnList[i];
                
                
                
                if("" === dgridColumn[0]){
                    columnId = dgridColumn[1];
                }else{
                    columnId = dgridColumn[0];
                }

                if(columnId === customViewColumnId){
                    index = i;
                    break;
                }
            }
            return index;
        },



        
        generateViewToDgridColumnMap: function(customViewColumns, dgridColumns){
            viewToDgridColumnMap = new Array();
            for(var i = 0;i < customViewColumns.length; i++){
                var index = this.findIndexByColumnId(dgridColumns, customViewColumns[i].id);
                if(index >= 0){
                    viewToDgridColumnMap[i] = index;
                }
            }
        },

        buildSNASheet: function () {
            
            
            var hiddenList = new Array();
            
            var snaRows = new Array();
            
            var tabsArray = new Array();

            var alertsViewController = this.alertsViewController;
            
            var checkedElements;

            
            var whichView = "alerts";
            var exportButton = registry.byId('exportbutton');
            if (exportButton && exportButton.mode !== undefined && exportButton.mode.length > 0) {
                whichView = exportButton.mode;
            }

            
            if (whichView === "alerts") { 
                checkedElements = alertsViewController.getSelectedAlerts();
            } else { 
                checkedElements = printAlertDetails(this.alertDetailsController);
            }


            var tableData = alertsViewController.getPrintData();

            
            
            
            var dgridColumns = new Array(new Array("__hidden"));

            
            if (whichView === "alerts") {
                
                dgridColumns = dgridColumns.concat(tableData.tables.tables.MEASURES.columns);

                
                
                var index = 1;
                array.forEach(dgridColumns, function (column) {
                    if (alertsViewController.isHiddenColumn(column)) {
                        hiddenList[hiddenList.length] = index;
                    }
                    index++;
                });
            } else { 
                
                
                tableData = checkedElements;
                
                
                var theTable = tableData.tables.tables;

                
                for (value in theTable) {
                    dgridColumns = dgridColumns.concat(theTable[value].columns);
                }
            }


            
            
            
            var customViewColumns = this.alertsViewController.model._d_dataGrid.gridPreferences().columns;
            this.generateViewToDgridColumnMap(customViewColumns, dgridColumns);

            
            var snaRow = new Object();
            snaRow.snaCells = _createSNACells(dgridColumns, hiddenList);
            snaRows[snaRows.length] = snaRow;

            var snaSheet = new Object();
            snaSheet.whichView = whichView;

            if (whichView === "alerts") { 
                snaSheet.elementsAreChecked = false;
                
                if (checkedElements.length > 0) {
                    var columns = alertsViewController.getHeaderLabels();
                    snaSheet.elementsAreChecked = true;

                    hiddenList[hiddenList.length] = 1;
                    var index = 1;
                    array.forEach(columns, function (column) {
                        if (alertsViewController.isHiddenColumn(column)) {
                            hiddenList[hiddenList.length] = index;
                        }
                        index++;
                    });

                    array.forEach(checkedElements, function (row) {
                        var snaCells = _createSNACells(row, hiddenList);
                        var snaRow = new Object();
                        snaRow.snaCells = snaCells;
                        snaRows[snaRows.length] = snaRow;
                    });

                } else { 
                    
                    
                    var dgrid = this.alertsViewController.model._d_dataGrid;
                    var table = dgrid.store.query(dgrid.get('query'), {sort:dgrid.get('sort')});
                    array.forEach(table, function (row) {
                        var snaRow = new Object();
                        snaRow.snaCells = _createSNACells(row, hiddenList);
                        snaRows[snaRows.length] = snaRow;
                    });
                }

                var currentVertical = ContextModel.get('currentAnalyticContext');
                snaSheet.snaRows = snaRows;
                snaSheet.fileName = currentVertical + ".xlsx";
                snaSheet.sheetName = currentVertical;

            } else { 
                for (tableTitle in theTable) {
                    var dgrid = this.alertDetailsController.model._alertTransactionDataGrids[tabsArray.length];
                    
                    
                    tabsArray[tabsArray.length] = JSON.parse(JSON.stringify(theTable[tableTitle]));
                    var currentTable = tabsArray[tabsArray.length - 1];
                    
                    var sortedData = dgrid.store.query(dgrid.get('query'), {sort:dgrid.get('sort')});
                    for(i = 0;i < currentTable.values.length; i++){
                        currentTable.values[i] = JSON.parse(JSON.stringify(sortedData[i]));
                    }
                    
                    if(currentTable.optional && currentTable.optional.tableDisplayName){
                        currentTable.displayName = currentTable.optional.tableDisplayName;
                    }
                    if (!currentTable.displayName) {
                        currentTable.displayName = tableTitle;
                    }
                    
                    if(currentTable.columns.length < currentTable.values[0].length){
                        
                        
                        
                        array.forEach(currentTable.values, function(row){
                            row.splice(0,1);
                        })
                    }

                    var customViewColumns = dgrid.gridPreferences().columns;
                    this.generateViewToDgridColumnMap(customViewColumns, currentTable.columns);

                    
                    
                    
                    
                    
                    var columnIsNumeric = [];
                    var columns = [];
                    var values = [];
                    for(var i = 0;i < currentTable.values.length;i++){
                        values.push([]);
                    }
                    viewToDgridColumnMap.forEach(function(columnIndex){
                       columnIsNumeric.push(currentTable.columnIsNumeric[columnIndex]);
                       var column = currentTable.columns[columnIndex];
                       if(!dgrid.columns[columnIndex].hidden){
                           columns.push(column);
                           currentTable.values.forEach(function (row, index) {
                               values[index].push(row[columnIndex])
                           });
                       }
                    });
                    currentTable.columnIsNumeric = columnIsNumeric;
                    currentTable.columns = columns;
                    currentTable.values = values;
                }

                
                var detailsInfo = this.alertDetailsController.getPrintData().detailsPropertyPanel.exportableDetails();
                if(detailsInfo){
                    tabsArray.unshift(detailsInfo);
                }


                var entity = getEntity(this.alertDetailsController);
                var fileName = entity.actionableEntityName;

                if (fileName.length < 1) {
                    fileName = entity.actionableEntityId;
                    if (fileName.length < 1) {
                        fileName = ContextModel.get('currentAnalyticContext');
                    }
                }

                snaSheet.fileName = fileName + ".xlsx";
                snaSheet.tabs = tabsArray;
            }


            return snaSheet;

            
            
            function _createSNACells(row, hiddenList) {
                var snaCells = new Array();
                viewToDgridColumnMap.forEach(function(columnIndex){
                     if (hiddenList.indexOf(columnIndex+1) == -1) {
                         var snaCell = new Object();
                         snaCell.value = new Array();
                         var cell = row[columnIndex];
                         
                         if (lang.isArray(cell)) {
                             snaCell.value = cell;
                         } else {
                             snaCell.value[snaCell.value.length] = cell;
                         }
                         snaCells[snaCells.length] = snaCell;
                     }

                });
                return snaCells;
            }

            function printAlertDetails(alertDetailsController) {
                return alertDetailsController.getPrintableTables();
            }

            function getEntity(alertDetailsController) {
                return alertDetailsController.getEntity();
            }

        },

        initiateDownload: function(snaSheetString){
            var that = this;
            var url = 'Export';
            
            
            

            console.log("Attempting to export = ", snaSheetString.length, " characters.");

            fileRequest = iframe.post(url, {
                handleAs: "xml",
                data: {snaSheet: snaSheetString},
                method: "POST",
                preventCache: true
            }).then(function (data) {
                    console.log("Export Result: ", data);
                    if("snaSheetIsNull" === data){
                        if(snaSheetString.length > maxExportCharacterSize){
                            MessageUtil.showErrorMessage(I18nUtil.get("error_export_data_too_large"));
                        }
                    }
                },
                function (error) {
                    console.log("Export Result: ", error);
                    if (error.name != "CancelError") {
                        console.log(error);
                    }
                });

            
            
            
            
            
            
            setTimeout(function(){
                fileRequest && fileRequest.cancel();
            }, 5000);

        }

});
});

               