
                   
define([
    'dojo/dom-class',
    'sna/util/I18nUtil',
    'sna/model/ContextModel',
    '../util/UserPreferenceUtil',
    'dojo/html',
    'dojo/_base/lang',
    './viewactions/DynamicUIUtil',
    'dojo/query',
    'dojo/on',
    './viewactions/DynamicUIRenderer',
    'dijit/registry',
    'sna/model/widgets/SNADialog',
    
    'dojo/text!src/sna/resources/html/templates/advancedsearch.html',
    'dojo/_base/declare',
    'dijit/form/NumberSpinner',
    'dojo/keys'
], function(domClass, I18nUtil, ContextModel, UserPreferenceUtil, html, lang, dynamicUIUtil, query, on, DynamicUIRenderer,
            registry, Dialog, template, declare , NumberSpinner, keys) {
	var preferenceKey = 'advancedsearch';
	
	var advanceSearchLocalValues = {};

    var savePreferences = function (vertical) {
        var enumeratedValues = this.renderer.enumerateValues();
        UserPreferenceUtil.retrieve('userpreferencewindow', vertical,function( pref ) {
            if (pref && pref.saveadvancesearchstatepreference && pref.saveadvancesearchstatepreference.value === 'on') {
                advanceSearchLocalValues[vertical] = enumeratedValues;
            } else {
                UserPreferenceUtil.save(preferenceKey, enumeratedValues, vertical);
            }
        });
        
    };

    var applyPreferences = function (vertical) {
        var that = this;
        
        UserPreferenceUtil.retrieve('userpreferencewindow', vertical,function( pref ) {
            if (pref && pref.saveadvancesearchstatepreference && pref.saveadvancesearchstatepreference.value === 'on') {
                var data = advanceSearchLocalValues[vertical];
                that.renderer.setValues(data);
            } else {
                UserPreferenceUtil.retrieve(preferenceKey, vertical, function (data) {
                    that.renderer.setValues(data);
                });
            }
        });
    };

    var resetValues = function () {
        
        var d_maxRowsSelect = registry.byId('advancedsearch-maxrows');
        d_maxRowsSelect.reset();
        this.renderer.reset();
    };

    var getToolbarButton = function () {
        return registry.byId('advancedsearchbutton');
    };

    return declare([], {
        
        constructor:function (args) {
            this.controller = args.controller;
            this.alertDetailsController = args.alertDetailsController;
            this.alertsViewController = args.alertsViewController;
        },

        
        render: function(vertical, uiDataMap, hideCloseButton) {
            var ui = uiDataMap.ui;
            var that = this;

            
            var d_dialog = new Dialog({
                title: uiDataMap.properties.title,
                content: template,
                _onChildChange: function() {
                    
                    
                }
            });
            domClass.add(d_dialog.domNode, "advancedsearchdialog");
            d_dialog.startup();
            
            if(hideCloseButton) {
                d_dialog.set('closable', false);
            }

            
            var d_content = registry.byNode(query('.content', d_dialog.domNode)[0]);
            this.renderer = new DynamicUIRenderer({
                d_context: d_content
            });
            this.renderer.render(ui);

            
            d_dialog.show();
            on(d_dialog, 'hide', function() {
                d_dialog.destroyDescendants();
                d_dialog.destroy();
            });
            
            on(d_dialog, 'cancel', function() {
            	if(hideCloseButton) {
            		
            		
            		that.controller.submissionFunction({}, vertical, {}, false);
            	}
            }); 
            
            
            var d_searchButton = registry.byId('advancedsearch-search');
            d_searchButton.setLabel(uiDataMap.properties.buttonLabel);
            on(d_searchButton, 'click', function () {
                lang.hitch(that, savePreferences, vertical)();
                that.startSearch(vertical, true);
                that.alertDetailsController.customView.clearCustomViewIndicator();
                that.alertsViewController.customView.clearCustomViewIndicator();
            });
            
            
            on(d_dialog, 'keydown', function(evt) {
                if ((evt.which && evt.which == keys.ENTER) || (evt.keyCode && evt.keyCode == keys.ENTER)) {
                    if (evt.target.nodeName != 'TEXTAREA') {
                        lang.hitch(that, savePreferences, vertical)();
                        that.startSearch(vertical, true);
                    }
                }
            });
            
            
            var _maxRowsValue = uiDataMap.properties.maxRowsValue;
            if(_maxRowsValue > uiDataMap.properties.maxRowsMax) {
                _maxRowsValue = uiDataMap.properties.maxRowsMax;
            }
            var d_maxRowsSelect = new NumberSpinner({
                value: _maxRowsValue
            },"advancedsearch-maxrows");
            d_maxRowsSelect.constraints = {
                max: uiDataMap.properties.maxRowsMax,
                min: uiDataMap.properties.maxRowsMin
            };
            on(d_maxRowsSelect, 'change', function() {
            	that.enforceConstraints(d_maxRowsSelect); 
            });
            
            
            on(d_maxRowsSelect,'keydown',function(evt) {
                var key = evt.keyCode;
                if ((evt.which && evt.which == keys.LEFT_ARROW) || (evt.keyCode && evt.keyCode == keys.LEFT_ARROW)
                        || (evt.which && evt.which == keys.RIGHT_ARROW) || (evt.keyCode && evt.keyCode == keys.RIGHT_ARROW)) {
                    evt.stopPropagation();
                }
            
            });
            
            d_maxRowsSelect.smallDelta = uiDataMap.properties.maxRowsStep || 1;


            this._d_maxRowsSelect = d_maxRowsSelect;

            
            var maxRowsLabel = query('#advancedsearch-maxrowslabel', d_dialog.domNode)[0];
            html.set(maxRowsLabel, uiDataMap.properties.maxRowsLabel);
            
            registry.byId('advancedsearch-restore').set('label', I18nUtil.get('preferences_restore'));

            this._d_dialog = d_dialog;
            this._groups = this.renderer.getGroups();
            this._data = uiDataMap;
            
            lang.hitch(that, applyPreferences, vertical)();
        },

        enforceConstraints: function(d_spinner) {
        	 var value =  d_spinner.get('value');
             if(value < d_spinner.constraints.min) {
             	value = d_spinner.constraints.min; 
             }
             else if (value > d_spinner.constraints.max) {
             	value = d_spinner.constraints.max; 
             }
             
             d_spinner.setValue(value); 
        },
        
        
        startSearch:  function (vertical) {
            var d_dialog = this._d_dialog;
            var parametersMap = {};
            var d_selectedTab = registry.byNode(query('.content', d_dialog.domNode)[0]).selectedChildWidget;

            parametersMap = dynamicUIUtil.generateParameters(d_selectedTab, d_selectedTab.id.replace('dynamic_', ''), this._groups, this._data.properties.concatDelimiter);

            
            var d_select = this._d_maxRowsSelect;
            this.enforceConstraints(d_select); 
            var value = d_select.get('value');

            
            
            ContextModel.set('currentAnalyticContext', vertical);
            
            parametersMap.queryParams.push({
                queryParameter: 'maxRows',
                queryType: 'GENERAL',
                queryValue: '%0'.tokenize(value)
            });
        
            this.controller.submissionFunction({}, vertical, parametersMap, true);
        },

        
        setUpPreferences:function () {
            var that = this;

            var restorePreferencesButton = registry.byId('advancedsearch-restore');
            on(restorePreferencesButton, 'click', function (e) {
                lang.hitch(that, resetValues)();
                UserPreferenceUtil.remove(preferenceKey);
            });
        },

        
        close: function() {
            this._d_dialog.destroyDescendants();
            this._d_dialog.destroy();
        },

        
        enableButton:function () {
            var toolbarButton = getToolbarButton();

            if (toolbarButton) {
                toolbarButton.set('disabled', false);
            }
        },

        
        disableButton:function () {
            var toolbarButton = getToolbarButton();

            if (toolbarButton) {
                toolbarButton.set('disabled', true);
            }
        },
        
        
        clearSavedPreferences:function (vertical,clearServer) {
            
            
            if (clearServer) {
                UserPreferenceUtil.retrieve(preferenceKey, vertical, function (data) {
                    advanceSearchLocalValues[vertical] = data;
                });
                UserPreferenceUtil.save(preferenceKey, new Object(), vertical);
            } else {
                UserPreferenceUtil.save(preferenceKey, advanceSearchLocalValues[vertical], vertical);
                advanceSearchLocalValues[vertical] = {};
            }
            
        }
    });
});

               