
                   
define([
    'dojo/has',
    'dojo/on',
    'fraudCommons/widgets/LoadingSpinner/LoadingSpinner',
    'sna/util/I18nUtil',
    'dojo/dom-construct',
    'dijit/_WidgetsInTemplateMixin',
    'dijit/_TemplatedMixin',
    'dijit/layout/BorderContainer',
    './Locator',
    'dojo/dom-style',
    'dojo/_base/lang',
    'dojox/lang/functional',
    'dojo/_base/array',
    'dijit/registry',
    './widgets/MeasuresTable',
    './viewactions/ObjectModelUtil',
    'dojo/_base/declare',
    'dojo/text!src/sna/resources/html/models/alertsview.html',
    'dojo/query',
    'dojo/ready'
], function (has, on, LoadingSpinner, I18nUtil, domConstruct, WidgetsInTemplateMixin, TemplatedMixin, BorderContainer, locator, domStyle, lang, functional, array, registry, MeasuresTable,
             objectModelUtil, declare, alertsView, query, ready) {

    var facetTypeName = 'alertsview';
    var that = this;

    
    var alertClicked = function (row, entities, vertical, submissions) {
        var entity = objectModelUtil.getActionableEntity(entities, objectModelUtil.getActionableEntityId(row));
        entity.measuresRow = row;
        this.controller.alertClicked(entity, vertical, submissions);
    };

    return declare([BorderContainer, TemplatedMixin, WidgetsInTemplateMixin], {
        templateString: alertsView,

        
        constructor: function (args) {
            this.inherited(arguments);

            this.controller = args.controller;
            this.d_context = args.d_context;
        },

        _getActionableEntities: function (d_dataGrid, checkedItems) {
            var entities = [];

            array.forEach(checkedItems, function (value, index) {
                var node = d_dataGrid.row(value);
                if (node) {
                    var actionableEntityId = d_dataGrid.dataGenerator.getActionableEntityId(node.data);
                    entities.push(d_dataGrid.dataGenerator.getActionableEntity(actionableEntityId));
                }
            });

            return entities;
        },

        destroyView: function () {
            registry.byNode(this.d_context).destroyDescendants();

            var d_facetContainer = locator.locate().alertsfacetcontainer();
            if (d_facetContainer) {
                d_facetContainer.collapse();
                d_facetContainer.hide();
            }

            this._updateCheckAllBox();
        },

        
        cleanupView: function (refreshing) {
            if (!refreshing) {
                var childNodes = registry.byNode(this.d_context).getChildren();
                array.forEach(childNodes, function (child) {
                    if (!child.alertDetailsFacetController) {
                        domStyle.set(child.domNode, "display", "none");
                    } else {
                        child.destroyRecursive();
                    }
                });
            } else {
                this.destroyView();
            }

            var d_facetContainer = locator.locate().alertsfacetcontainer();
            if (d_facetContainer && domStyle.get(d_facetContainer.getParent().domNode, "display") !== "none") {
                d_facetContainer.collapse();
                d_facetContainer.hide();
            }
        },

        unhideView: function () {
            var childNodes = registry.byNode(this.d_context).getChildren();
            array.forEach(childNodes, function (child) {
                domStyle.set(child.domNode, "display", "block");
            });
            registry.byNode(this.d_context).getParent().layout();
            this.setDispositionButtonState();
        },

        setDispositionButtonState: function () {
            var numAlertsChecked = query('.dgrid-content input[type=checkbox][aria-checked="true"]:checked', this._d_dataGrid.domNode).length;
            if (numAlertsChecked > 0) {
                this.controller.dispositionController.enable();
            } else {
                this.controller.dispositionController.disable();
            }
        },

        _buildDispositionList: function(event, object, d_checkbox) {
            var _this = this;
            var checked = this._getCheckedItems();
            if (checked.length > 0) {
                _this.controller.dispositionController.enable();
            } else {
                _this.controller.dispositionController.disable();
            }
            var entities = _this._getActionableEntities(_this._d_dataGrid, checked);
            _this.controller.updateDispositionItemsList(entities, 'alerts');
            _this._updateCheckAllBox();
        },

        getCheckedEntities: function () {
            if (!this._d_dataGrid) return [];
            var checked = query('.dgrid-content input[type=checkbox][aria-checked="true"]:checked', this._d_dataGrid.domNode);
            var count = checked.length;
            
            return this._getActionableEntities(this._d_dataGrid, checked);
        },


        
        _updateCheckAllBox: function () {
            
            if (this._d_dataGrid && this._d_dataGrid.checkAllCheckbox) {
                var numEntitiesShown;
                var checked = this._getCheckedItems();
                var d_dataGrid = this._d_dataGrid;
                var checkAllCheckbox = d_dataGrid.checkAllCheckbox;

                if (d_dataGrid._total < d_dataGrid.rowsPerPage) {
                    numEntitiesShown = d_dataGrid._total;
                    
                } else if ((d_dataGrid && d_dataGrid.store
                    && d_dataGrid.store.data
                    && d_dataGrid.store.data.length)
                    && (d_dataGrid.store.data.length < d_dataGrid.rowsPerPage)) {
                    numEntitiesShown = d_dataGrid.store.data.length;
                } else {
                    numEntitiesShown = d_dataGrid._rowsOnPage;
                }

                checkAllCheckbox.set('checked', checked.length === numEntitiesShown);
            }
        },

        
        
        deleteGridPrefs: function (name) {
            this._d_dataGrid.deleteGridPrefs(name);
        },
        
        deleteAllGridPrefs: function () {
            this._d_dataGrid.deleteAllGridPrefs();
        },

        saveGridPrefs: function (name) {
            this._d_dataGrid.saveGridPrefs(name);
        },

        applyGridPrefs: function (name) {
            this._d_dataGrid.applyGridPrefs(name);
        },

        _getCheckedItems: function () {
            if (has("ie") >= 9) {
                return query('.dgrid-scroller input[type=checkbox][aria-checked="true"]:checked', this._d_dataGrid.domNode);
            } else {
                return query('.dgrid-scroller input[type=checkbox]:checked', this._d_dataGrid.domNode);
            }
        },

        _resetDispostionList: function () {
            if (this._d_dataGrid) {
                this.controller.updateDispositionItemsList([], 'alerts');
            }
        },

        
        render: function (sVertical, alertedEntities, submissions, showFacetPanel) {
            var that = this;

            var completedLoadingFlag = false;


            this._searchParameters = submissions;

            domConstruct.place(this.domNode, this.d_context);

            var d_facetContainer = this.alertsviewfacetspanel;
            
            if (showFacetPanel !== 'y') { 
                d_facetContainer.set('initialState', 'collapsed');
                d_facetContainer.set('splitter', false);
            }
            else if (alertedEntities.facets.length === 0) {
                d_facetContainer.set('initialState', 'collapsed');
            }
            else {
                d_facetContainer.expand();
            }

            this.startup();
            this.resize();
            this.layout();

            domStyle.set(this.noalertsmessage, 'display', 'none');

            var d_dataGrid;
            var grid = new MeasuresTable({
                data: alertedEntities,
                onchange: lang.hitch(that, that._buildDispositionList),
                updateCheckAllBox: lang.hitch(that, that._updateCheckAllBox),
                resetDispostionList: lang.hitch(that, that._resetDispostionList),
                options: {
                    useCheckboxes: true,
                    selectionMode: 'single',
                    customView: that.controller.customView
                },
                tooltipsEnabled: true
            });

            var filterQueryFunction = this.controller.getFilterQueryFunction();
            d_dataGrid = grid.render('MEASURES', this.grid, false, {
                    ondblclick: function (row) {
                        lang.hitch(that, alertClicked, row.data, alertedEntities.actionableEntities, sVertical, submissions)();
                    }
                }, {
                    enableSelection: true
                },
                filterQueryFunction
            );
            this._d_dataGrid = d_dataGrid;
            this._printData = alertedEntities;

            this.alertsviewfacetspanel.set('title', I18nUtil.get('alertsview_filters'));

            ready(function () {
                that.controller.startDispositionController(sVertical, function () {
                    
                    that.controller.updateDispositionItemsList([], 'alerts');
                });

                that.controller.startExternalAppController(sVertical, function () {
                    that.controller.updateExternalAppsList([], 'alerts');
                });

                completedLoadingFlag = true;
            });
        },

        refreshDataGrid: function () {
            if (this._d_dataGrid) {
                this._d_dataGrid.refresh();

                var checked = this._getCheckedItems();
                var entities = this._getActionableEntities(this._d_dataGrid, checked);
                this.controller.updateDispositionItemsList(entities, 'alerts');

                this._d_dataGrid.checkAllCheckbox.set('checked', false);
            }
        },

        dispositionCompleted: function () {

        },

        getPrintData: function () {
            return {
                tables: this._printData
            };
        },

        getAdvancedSearchParameters: function () {
            return this._searchParameters;
        },

        
        getSelectedAlerts: function () {
            console.log('alertsviewmodel.getSelectedAlerts', this._d_dataGrid);
            var d_dataGrid = this._d_dataGrid;
            var alertedEntities = new Array();
            var checkedElements = query('.dgrid-scroller input[type=checkbox][aria-checked="true"]:checked', d_dataGrid.domNode);
            array.forEach(checkedElements, function (value) {
                var d_row = d_dataGrid.row(value);
                alertedEntities[alertedEntities.length] = d_dataGrid.store.get(d_row.data[0]);
            });
            return alertedEntities;
        },

        
        isHiddenColumn: function (column) {
            var index = this.controller.getColumnIndexFromColumnIdentifier(column[0]);
            var d_dataGrid = this._d_dataGrid;
            if (objectModelUtil.isColumnHidden(column))
                return true;
            return d_dataGrid.isColumnHidden(index);
        },

        
        getHeaderLabels: function () {
            var d_dataGrid = this._d_dataGrid;
            var columns = d_dataGrid.columns;
            var columnLabels = new Array();
            for (var i = 0; i < Object.keys(columns).length; i++) {
                column = columns[i];
                columnLabels[columnLabels.length] = [column.field, column.label];
            }
            return columnLabels;
        },

        
        getColumnIndexFromColumnIdentifier: function (columnIdentifier) {
            var index = -1;

            var that = this;
            if (this.isDGridReady()) {
                that._d_dataGrid._columnsMap = that._d_dataGrid._columnsMap || {};

                if (that._d_dataGrid._columnsMap[columnIdentifier]) {
                    return that._d_dataGrid._columnsMap[columnIdentifier];
                }

                functional.forIn(that._d_dataGrid.columns, function (column, count) {
                    if (column.field === columnIdentifier) {
                        index = count;
                        that._d_dataGrid._columnsMap[columnIdentifier] = count;
                    }
                });
            }
            return index;
        },

        isDGridReady: function () {
            if (this._d_dataGrid) {
                return true;
            }

            return false;
        },

        destroyDataGrid: function () {
            if (this._d_dataGrid) {
                if (this._d_dataGrid.destroy) {
                    this._d_dataGrid.destroy();
                }
                this._d_dataGrid = undefined;
            }
        },

        
        removeCheckedRows: function () {
            var d_dataGrid = this._d_dataGrid;
            var checkedElements = query('.dgrid-scroller input[type=checkbox][aria-checked="true"]:checked', d_dataGrid.domNode);

            console.log('AlertsViewModel.removeCheckedRows', d_dataGrid);

            this.removeRows(checkedElements);
        },

        removeRows: function (elements) {
            var d_dataGrid = this._d_dataGrid;

            array.forEach(elements, function (value) {
                value = value.actionableEntityId || value;
                var d_row = d_dataGrid.row(value);
                console.log('removing ', d_row);
                d_dataGrid.store.remove(d_row.data[0]);
            });
        }
    });
});

               