
                   
define([
    'dojo/dom-style',
    'fraudCommons/util/NLSUtil',
    'dojo/i18n!../util/nls/validate',
    './widgets/Table',
    'dojo/_base/array',
    'dojo/on',
    'dijit/_WidgetBase',
    'dijit/registry',
    'dijit/form/CheckBox',
    'dijit/form/NumberSpinner',
    'dojo/dom-construct',
    'dojo/_base/declare',
    'dojo/text!src/sna/resources/html/templates/exportDialog.html',
    'dijit/Dialog',
    'dojo/query'
], function (domStyle, NLSUtil, validate, TableWidget,
             array,
             on, _WidgetBase, registry, CheckBox, NumberSpinner, domConstruct, declare, template, Dialog, query) {

    var rowHeaderLabel = function(){
        return localizedLabel('export_rows');
    };
    var localizedLabel = function(label){
        return domConstruct.toDom('<b><label for="key">%0</label></b> '.tokenize(NLSUtil.resolve(label, validate)));
    };
    var blankLabel = function(){
        return domConstruct.toDom('<label for="key"></label> ');
    };

    
    var areAllCheckBoxesChecked = function () {
        var checkBoxes = query('.export-options-items .dijitCheckBox');

        if (checkBoxes) {
            return checkBoxes.every(function (d_cb) {
                var checkBox = registry.byNode(d_cb);
                if (checkBox.get('name') === "__all__") {
                    return true;
                }
                return checkBox.get('checked');
            });
        } else {
            return true;
        }
    };


    var updateCheckboxesBasedOnCheckAllState = function (dialog, allCheckbox) {
        var checkBoxes = query('.export-options-items .dijitCheckBox');
        var checkAllChecked = allCheckbox.get('checked');

        if (checkBoxes) {
            
            checkBoxes.forEach(function (d_cb) {
                var checkBox = registry.byNode(d_cb);
                if (checkBox.name !== "__all__") {
                    checkBox.set('checked', checkAllChecked);

                    if (!checkBox.get('checked')) {
                        dialog.excludedItems.add(checkBox.name);
                    } else {
                        dialog.excludedItems.remove(checkBox.name);
                    }
                }
            });
        }
    };

    var addAllCheckbox = function (dialog) {
        
        
        if (!dialog.snaSheet || !dialog.snaSheet.tabs || (dialog.snaSheet.tabs.length < 2)) {
            dialog.tableWidgetRow = dialog.tableWidget.addColumn(dialog.tableWidgetRow, blankLabel()).row;
            dialog.tableWidgetRow = dialog.tableWidget.addColumn(dialog.tableWidgetRow, rowHeaderLabel()).row;
            dialog.tableWidgetRow = dialog.tableWidget.addColumn(dialog.tableWidgetRow, blankLabel()).row;
            return;

        }
        
        dialog.checkAll = new CheckBox({
            value: "__all__",
            name: "__all__",
            "class": "allCheckbox",
            checked: dialog.excludedItems.isEmpty()
        });

        dialog.tableWidgetRow = dialog.tableWidget.addColumn(dialog.tableWidgetRow, dialog.checkAll.domNode).row;
        domConstruct.place(domConstruct.toDom('<label for="key">%0</label> '.tokenize(NLSUtil.resolve("export_select_all", validate))), dialog.checkAll.domNode, 'after');
        dialog.tableWidgetRow = dialog.tableWidget.addColumn(dialog.tableWidgetRow, rowHeaderLabel()).row;
        dialog.tableWidgetRow = dialog.tableWidget.addColumn(dialog.tableWidgetRow, blankLabel()).row;

        on(dialog.checkAll, 'click', function (e) {
            updateCheckboxesBasedOnCheckAllState(dialog, this);
        });

    };

    var recallSpinnerValue = function(dialog, defaultValue, displayName, spinnerName){
        if(shouldRecallLastExport(dialog)){
            return dialog.lastExport.tableSettings[displayName][spinnerName];
        }else{
            return defaultValue;
        }
    };

    var addDetailTableRow = function (dialog, tab) {
        var optionId = tab.displayName;
        var numRows = tab.values.length;
        var optionLabel = optionId;
        if (tab.optional && tab.optional.tableDisplayName) {
            optionLabel = tab.optional.tableDisplayName;
        }

        if(dialog.snaSheet.tabs.length > 1){
            var checkBox = new CheckBox({
                value: optionId,
                name: optionId,
                checked: !dialog.excludedItems.contains(optionId)
            });
            dialog.tableWidgetRow = dialog.tableWidget.addColumn(dialog.tableWidgetRow, checkBox.domNode,undefined,"tableName").row;
            
            var label = domConstruct.toDom('<label for="key">%0</label> '.tokenize(optionLabel));
            domConstruct.place(label, checkBox.domNode, 'after');
        }else{
            dialog.tableWidgetRow = dialog.tableWidget.addColumn(dialog.tableWidgetRow, domConstruct.toDom('<label for="key">%0</label> '.tokenize(optionLabel)),undefined,"tableName").row;
        }
        tab.startRowSpinner = new NumberSpinner({
            value: recallSpinnerValue(dialog,1,tab.displayName, "rowStart"),
            smallDelta: 1,
            constraints: {min: 1, max: numRows, places: 0},
            id: "startRowSpinnerFor" + optionLabel,
            style: "width: 80px"
        });
        dialog.tableWidgetRow = dialog.tableWidget.addColumn(dialog.tableWidgetRow, tab.startRowSpinner.domNode).row;
        tab.endRowSpinner = new NumberSpinner({
            value: recallSpinnerValue(dialog,numRows,tab.displayName, "rowEnd"),
            smallDelta: 1,
            constraints: {min: 1, max: numRows, places: 0},
            id: "endRowSpinnerFor" + optionLabel,
            style: "width: 80px"
        });
        dialog.tableWidgetRow = dialog.tableWidget.addColumn(dialog.tableWidgetRow, tab.endRowSpinner.domNode).row;

        if(dialog.snaSheet.tabs.length > 1) {
            on(checkBox, 'click', function () {
                dialog.checkAll.set('checked', areAllCheckBoxesChecked());
                if (!checkBox.checked) {
                    dialog.excludedItems.add(optionId);
                }
                else {
                    dialog.excludedItems.remove(optionId);
                }
            });
        }
    };

    var placeTableWidgetInOptionsInContainer = function (dialog) {
        var exportControlWidget = registry.byNode(query('.export-controls', dialog.parentWidget.domNode)[0]);
        var container = query('.export-options-container', exportControlWidget.domNode)[0];
        var optionsContainer = query('.export-options-items', container)[0];

        domConstruct.place(dialog.tableWidget.getTable().domNode, optionsContainer);
    };

    
    var createOptionsTableWidget = function (dialog) {
        dialog.tableWidget = new TableWidget({
            maxColumns: 3
        });

        dialog.tableWidgetRow = dialog.tableWidget.addRow(0);
    };

    var isAlertView = function (dialog) {
        return "alerts" === dialog.snaSheet.whichView;
    };

    var rowStart = function(table){
        return table.startRowSpinner.get("value");
    };
    var rowEnd = function(table){
        return table.endRowSpinner.get("value");
    };

    var clearSpinnerValues = function (table) {
        table.startRowSpinner = undefined;
        table.endRowSpinner = undefined;
    };

    var headerRow = function(array){
        return array.slice(0,1);
    };

    var removeUnneededRows = function (dialog) {
        var sheet = dialog.snaSheet;
        if (isAlertView(dialog)) {
            sheet.snaRows = headerRow(sheet.snaRows).concat(sheet.snaRows.slice(rowStart(sheet), rowEnd(sheet) + 1));
        } else {
            sheet.tabs.forEach(function (tab) {
                tab.values = tab.values.slice(rowStart(tab)- 1,
                    rowEnd(tab));
            });
        }
    };

    var findTabWithName = function(list, name){
        var index;
        for (var i = 0; i < list.length; ++i) {
            if (list[i].displayName === name) {
                index = i;
                break;
            }
        }
        return index;
    };

    var removeUnneededTables = function (dialog) {
        if (isAlertView(dialog)) {
            return;
        }
        dialog.excludedItems.itemList.forEach(function (item) {
            var tabIndex = findTabWithName(dialog.snaSheet.tabs, item);
            dialog.snaSheet.tabs.splice(tabIndex, 1);
        });
    };

    var saveSpinners = function(dialog){
        dialog.lastExport.tableSettings = [];
        if (isAlertView(dialog)) {
            var settings = {};
            settings.rowStart = rowStart(dialog.snaSheet);
            settings.rowEnd = rowEnd(dialog.snaSheet);
            dialog.lastExport.tableSettings["alerts"] = settings;
        } else {
            dialog.snaSheet.tabs.forEach(function (tab) {
                var settings = {};
                settings.rowStart = rowStart(tab);
                settings.rowEnd = rowEnd(tab);
                dialog.lastExport.tableSettings[tab.displayName] = settings;
            });
        }
    };

    
    var clearSpinners = function(dialog){
        if (isAlertView(dialog)) {
            clearSpinnerValues(dialog.snaSheet);
        } else {
            dialog.snaSheet.tabs.forEach(function (tab) {
                clearSpinnerValues(tab);
            });
        }
    };

    var rememberLastExport = function (dialog) {
        var sheet = dialog.snaSheet;
        dialog.lastExport.fileName = sheet.fileName;
        dialog.lastExport.excludedItemsList = dialog.excludedItems.itemList;
        dialog.lastExport.view = sheet.whichView;
        saveSpinners(dialog);
    };

    
    
    var shouldRecallLastExport = function (dialog) {
        var sheet = dialog.snaSheet;
        return (sheet.whichView === dialog.lastExport.view) &&(sheet.fileName === dialog.lastExport.fileName);
    };


    
    return declare(_WidgetBase, {

        
        constructor: function (args) {
            declare.safeMixin(this, args);

            var that = this;
            that.parentWidget = new Dialog({
                title: NLSUtil.resolve('export_title', validate),
                content: template,
                id: 'export-dialog',
                onHide: function () {
                    rememberLastExport(that);
                    that.cleanupView();
                },
                "class": 'exportDialog'
            });

            
            

            on(query('#export-button'), 'click', function () {
                rememberLastExport(that);
                removeUnneededTables(that);
                removeUnneededRows(that);
                clearSpinners(that);
                that.exportCallback(JSON.stringify(that.snaSheet));
                that.cleanupView();
            });
        },


        generateDetailsOptions: function () {
            var dialog = this;
            if(shouldRecallLastExport(dialog)){
                this.excludedItems.initialize(this.lastExport.excludedItemsList);
            }else{
                this.excludedItems.reset();
            }

            addAllCheckbox(this, this.snaSheet.tabs);
            dialog.snaSheet.tabs.forEach(function (tab) {
                addDetailTableRow(dialog, tab);
            });

        },


        generateAlertsOptions: function () {
            var dialog = this;

            dialog.tableWidgetRow = dialog.tableWidget.addColumn(dialog.tableWidgetRow, blankLabel()).row;
            dialog.tableWidgetRow = dialog.tableWidget.addColumn(dialog.tableWidgetRow, rowHeaderLabel()).row;
            dialog.tableWidgetRow = dialog.tableWidget.addColumn(dialog.tableWidgetRow, blankLabel()).row;

            
            
            var label = domConstruct.toDom('<label>'+ this.snaSheet.sheetName + '</label> ');
            dialog.tableWidgetRow = dialog.tableWidget.addColumn(dialog.tableWidgetRow, label,undefined,"tableName").row;


            var numRows = this.snaSheet.snaRows.length - 1;
            this.snaSheet.startRowSpinner = new NumberSpinner({
                value: recallSpinnerValue(dialog,1, "alerts", "rowStart"),
                smallDelta: 1,
                constraints: {min: 1, max: numRows, places: 0},
                id: "startRowSpinnerForAlerts",
                style: "width: 80px"
            });
            dialog.tableWidgetRow = dialog.tableWidget.addColumn(dialog.tableWidgetRow, this.snaSheet.startRowSpinner.domNode).row;
            this.snaSheet.endRowSpinner = new NumberSpinner({
                value: recallSpinnerValue(dialog,numRows, "alerts", "rowEnd"),
                smallDelta: 1,
                constraints: {min: 1, max: numRows, places: 0},
                id: "endRowSpinnerForAlerts",
                style: "width: 80px"
            });
            dialog.tableWidgetRow = dialog.tableWidget.addColumn(dialog.tableWidgetRow, this.snaSheet.endRowSpinner.domNode).row;
        },

        
        show: function (snaSheet, exportCallback, lastExport) {
            this.snaSheet = snaSheet;
            this.exportCallback = exportCallback;
            this.parentWidget.show();
            this.lastExport = lastExport;
            createOptionsTableWidget(this);
            if (isAlertView(this)) {
                this.generateAlertsOptions();
            } else {
                this.generateDetailsOptions();
            }
            placeTableWidgetInOptionsInContainer(this);

            registry.byId('export-button').set('label', NLSUtil.resolve('export_save', validate));
        },


        
        cleanupView: function () {
            if (this.parentWidget) {
                this.parentWidget.destroyRecursive();
            }
        },

        
        excludedItems: {
            itemList: [],
            add: function (item) {
                this.itemList.push(item);
                this.isChanged = true;
            },
            remove: function (storedItem) {
                this.itemList = array.filter(this.itemList, function (soughtItem) {
                    return soughtItem !== storedItem;
                });
                this.isChanged = true;
            },
            isChanged: false,
            contains: function (item) {
                return this.itemList.indexOf(item) > -1;
            },
            isEmpty: function () {
                return this.itemList.length < 1;
            },
            initialize: function (itemList) {
                this.itemList = itemList;
                this.isChanged = true;
            },
            reset: function () {
                this.itemList = [];
                this.isChanged = true;
            }
        },

        
        columnHiddenStates: {}
    });
});

               