
                   
define([
    './MultiColumnSortingQueryEngine',
    'dojo/dom-construct',
    'dojo/mouse',
    'dijit/Tooltip',
    'dojo/string',
    'dojo/_base/lang',
    'dojo/on',
    'dojo/number',
    '../widgets/KPIIndicator',
    'dojo/_base/declare',
    'dojo/_base/array',
    'dojo/store/Memory',
    'sna/model/widgets/AlertSeverityIndicator',
    'dojo/dom-attr',
    'dojo/NodeList-traverse'
], function(MultiColumnSortingQueryEngine, domConstruct, mouse, Tooltip, string, lang, on, number, KPIIndicator, declare, array, Memory, AlertSeverityIndicator, domAttr) {
    var _getActionableEntity = function(entities, id) {
        return array.filter(entities, function(item) {
            return item.actionableEntityId === id;
        })[0];
    };

    
    var _getRowIdentity = function (row) {
        return row[0][0];
    };

    var _tableHasActionableEntities = function(tableData) {
        if(tableData.values.length > 0 && tableData.values[0].length > 0) {
            return tableData.values[0][0].length === 1;
        }

        return false;
    };


    
    
    var _getRawValue = function(value) {
        if(value[0] === "") {
            return value[1];
        }
        else {
            return value[0];
        }
    };

    var _getFormattedValue = function(value) {
        if(value.length === 1) {
            return ''; 
        }

        if(value[1] === null) {
            return null;
        }

        
        if(value[1].indexOf('\\u') !== -1) {
            return JSON.parse('"%0"'.tokenize(value[1]));
        }

        return value[1];
    };

    
    var getCellRenderer = function(columnObject, column, columnIndex, other, style) {
        return function(object, value, node, options) {
            var that = this;
            var columnData = object[columnIndex];
            var rawValue = _getRawValue(columnData);
            var formattedValue = _getFormattedValue(columnData);

            var decorateNode = function(node) {
                if(columnIndex === 1) {
                    if(other && other.actionableEntities) {
                        domAttr.set(node, 'data-id', _getActionableEntity(other.actionableEntities, _getRowIdentity(object)).actionableEntityId);
                    }
                }
            };

            var columnStyle  = columnData[2] + style;

            
            if(columnData.length === 3) {
                var newNode = domConstruct.toDom('<div style="%0">'.tokenize(columnStyle) + formattedValue + '</div>');
                decorateNode(newNode);
                return newNode;
            }

            
            var special = columnData[3];

            var component;

            if(special.type === 'severity') {
                component = new AlertSeverityIndicator({value: columnData[0], max: special.max, min: special.min, median: special.median}).placeAt(node);
            }
            else if (special.type === 'kpi') {
                component = new KPIIndicator({target: special.target, score: special.score}).placeAt(node);
            }
            else if (special.url) {
                 
                special.url = special.url.replace(/%26/g, '&');
                
                
                
                var linkNode = domConstruct.toDom(string.substitute('<a href="${link}" target="_blank">${text}</a>', {
                    link: special.url,
                    text: formattedValue
                }));

                var replacementString = '<div style="${style}"></div>';
                replacementString = string.substitute(replacementString, {
                    style: columnStyle,
                    link: special.url,
                    text: formattedValue
                }).replace(/%26/g, '&');

                var urlNode = domConstruct.toDom(replacementString);
                domConstruct.place(linkNode, urlNode);

                on(linkNode, 'click', function(event) {
                    event.stopPropagation();
                    that.grid.deselect(event);
                });

                domConstruct.place(urlNode, node) ;
            }
            else if (special.type === 'image') {
                var imageNode = domConstruct.toDom('<div style="%0"><img src="%1"></div>'.tokenize(columnStyle, special.src));
                domConstruct.place(imageNode, node);
            }

            if(component) {
                
                this.grid.itemsToDestroy = this.grid.itemsToDestroy || [];
                this.grid.itemsToDestroy.push(component);
                component.placeAt(node);
            }

            decorateNode(node);
        };
    };

    var _columnIsHidden = function(column) {
        return _getRawValue(column) === null ||  _getRawValue(column).startsWith('__') || (column.length > 3 && typeof(column[3].hidden) !== 'undefined' && column[3].hidden === true);
    };

    return {
        
        getDataGenerator: function(table, data) {
            var tableData = data.tables[table];
            var that = this;

            
            
            var increment = 0;
            if(_tableHasActionableEntities(tableData)) {
                increment = 1;
            }

            
            
            
            tableData.columnIsNumeric = [];
            array.forEach(tableData.columns, function(colVal, colIndex) {
                tableData.columnIsNumeric[colIndex] = array.every(tableData.values, function(row, rowIndex) {
                    var value = that.getRawValue(row[colIndex+increment]);
                    
                    return isNumber(value) || value.trim() === "";
                });
            });


            
            var CustomMemory = declare([Memory], {
                queryEngine: MultiColumnSortingQueryEngine,

                
                _comparisonFunction: function(a, b, column) {
                    var columnIndex = 0;

                    
                    array.forEach(tableData.columns, function(value, index) {
                        
                        
                        if(that.getRawColumnValue(value) === column) {
                            columnIndex = index;
                            
                            
                            return;
                        }
                    });

                    
                    var first = that.getRawValue(a[columnIndex + increment]);
                    var second = that.getRawValue(b[columnIndex + increment]);

                    if(tableData.columnIsNumeric[columnIndex]) {
                        
                        
                        if((first.trim() === "") || (second.trim() === "")){
                            return first.localeCompare(second);
                        }
                        
                        
                        
                        
                        
                        return number.parse(first,{locale:"en-us"}) - number.parse(second,{locale:"en-us"});
                    }

                    return first.localeCompare(second);
                },

                
                setData: function(data){
                    this.data = data;
                    this.index = {};
                    for(var i = 0, l = data.length; i < l; i++){
                        this.index[this.getIdentity(data[i])] = i;
                    }
                },
                getIdentity: function(object) {
                    return _getRowIdentity(object);
                },

                
                query:function(query,options){
                    var compare = this._comparisonFunction;

                    
                    if(options.sort!=undefined && options.sort.length > 0){
                        options.custom = [];
                        array.forEach(options.sort, function(sort, index) {
                            sort.compare = function(a, b) {
                                return compare(a, b, sort.attribute);
                            }
                        });

                        return this.inherited(arguments);
                    }

                    return this.inherited(arguments);
                }
            });

            
            var rowsStore = new CustomMemory({
                data:tableData.values
            });

            return {
                columns: function() {
                    var index = 0;
                    var mappedColumns = array.map(tableData.columns, function(column) {
                        var newColumn = (function(i) {
                            var columnObject = {};
                            lang.mixin(columnObject, {
                                field: _getRawValue(column),
                                label: _getFormattedValue(column),
                                get: function(element) {
                                    
                                    return element[i+increment][1];
                                },
                                renderCell: getCellRenderer(columnObject, column, i+increment, {actionableEntities: data.actionableEntities}, column[2]),
                                renderHeaderCell: function(node) {
                                    var columnStyle = '';
                                    if(column[3] && column[3].headerStyle) {
                                        columnStyle = column[3].headerStyle;
                                    }

                                    if (!this.alreadySized) {  
                                        if(columnObject._previousColumnWidth) {
                                            this.grid.styleColumn(this.id, 'width: %0px;'.tokenize(columnObject._previousColumnWidth));
                                            this.alreadySized = true;
                                        }
                                        else if(column[3] && column[3].minWidth) {
                                            this.grid.styleColumn(this.id, 'width: %0px;'.tokenize(column[3].minWidth));
                                            this.alreadySized = true;
                                        }
                                        else {
                                            this.grid.styleColumn(this.id, 'width: %0px;'.tokenize(200));
                                            this.alreadySized = true;
                                        }
                                    }

                                    return domConstruct.toDom('<div style="%0">%1</div>'.tokenize(columnStyle, column[1]));
                                },
                                hidden: _columnIsHidden(column),
                                unhidable: _columnIsHidden(column)
                            });

                            return columnObject;
                        }(index));
                        index += 1;

                        return newColumn;
                    });

                    return mappedColumns;
                },
                rows: function() {
                    return tableData.values;
                },
                rowsStore: function() {
                    return rowsStore;
                },
                getActionableEntity: function(id) {
                    return _getActionableEntity(data.actionableEntities, id);
                },
                getActionableEntityId: function (row) {
                    return _getRowIdentity(row);
                },
                getRawTableData: function() {
                    return tableData;
                }
            };
        },

        
        getActionableEntity:  function(entities, id) {
            return _getActionableEntity(entities, id);
        },

        
        getActionableEntityId: function(row) {
            return _getRowIdentity(row);
        },

        
        getRawColumnValue: function(column) {
            if(column[0] === "") {
                return column[1];
            }
            else {
                return column[0];
            }
        },

        
        getRawValue: function(value) {
            return _getRawValue(value);
        },

        
        getFormattedValue: function(value) {
            return _getFormattedValue(value);
        },

        
        getColumnStyle: function(column) {
            return column[2];
        },

        
        getSpecialColumnProperties: function(column) {
            return column[3] || {};
        },

        
        getColumnIndex: function(row, index) {
            if(row[0].length === 1) {
                return (1 + number.parse(index));
            }

            return (0 + number.parse(index));
        },
        
        
        isColumnHidden: function(column) {
            return _columnIsHidden(column);
        }
    };
});
               