
                   
define([
    'sna/util/marked/marked',
    'dgrid/util/touch',
    'fraudCommons/util/GridPrefs',
    './MultiColumnSortingMixin',
    './ColumnReorderWidthPreserveMixin',
    'dojo/dom-construct',
    'dojo/Deferred',
    'dijit/registry',
    '../../util/I18nUtil',
    'dojo/i18n!dgrid/extensions/nls/pagination',
    'dojo/dom-attr',
    'dojo/dom-style',
    'dojo/dom-geometry',
    'dojo/_base/array',
    'dojo/query',
    'dijit/form/RadioButton',
    'dijit/Tooltip',
    'dojo/_base/lang',
    'dojo/dom-style',
    'dojo/mouse',
    'dojo/on',
    'dijit/form/CheckBox',
    'dgrid/extensions/ColumnResizer',
    'dgrid/extensions/ColumnHider',
    'dgrid/extensions/Pagination',
    'dgrid/Keyboard',
    'dojo/_base/declare',
    'dojox/lang/functional',
    'dgrid/Grid',
    '../viewactions/ObjectModelUtil',
    'dgrid/Selection',
    'dgrid/extensions/DijitRegistry'
], function (marked, touch, GridPrefs, MultiColumnSortingMixin, ColumnReorderWidthPreserveMixin, domConstruct, Deferred, registry, I18nUtil, i18nPagination,
             domAttr, domStyle, domGeometry, array, query, RadioButton, Tooltip, lang, style, mouse, on, CheckBox, ColumnHider, ColumnResizer, Pagination,
             Keyboard, declare, functional, Grid, objectModelUtil, Selection, DijitRegistry) {
    var getPageSizes = function (dataLength) {
        if (dataLength <= 25) {
            return [dataLength];
        }
        else if (dataLength <= 50) {
            return [25, dataLength];
        }
        else if (dataLength <= 100) {
            return [25, 50, dataLength];
        }
        else if (dataLength <= 250) {
            return [25, 50, 100, dataLength];
        }
        else if (dataLength <= 500) {
            return [25, 50, 100, 250, dataLength];
        }
        else {
            return [25, 50, 100, 250, 500, dataLength];
        }
    };

    return declare("sas.sna.model.widgets.MeasuresTable", [], {
        
        constructor: function (args) {
            var that = this;
            
            this._rawData = args.data;
            this.tooltips = {};
            this._oncheck = args.oncheck;
            this._onchange = args.onchange;
            this.updateCheckAllBox = args.updateCheckAllBox;
            this.resetDispostionList = args.resetDispostionList;
            this._options = args.options;
            this.useCheckboxes = args.useCheckboxes;

            
            functional.forIn(this._rawData.tables, function(table, tableKey) {
                that.tooltips[tableKey] = {};
                var colsToDelete = [];
                table.columns.forEach(function (column, i) {
                    
                    var regex = /(.*)_tooltip$/gi;
                    var match = regex.exec(column[0]);
                    if (match && match.length >= 2) {
                        that.tooltips[tableKey][match[1]] = {};
                        colsToDelete.push(i);
                        table.values.forEach(function (value, j) {
                            
                            if(value[0].length === 1) {
                                that.tooltips[tableKey][match[1]][value[0]] = value[i + 1];
                            } else {
                                that.tooltips[tableKey][match[1]][value[0][0]] = value[i];
                            }
                        });
                    }
                });

                
                var colShiftFlag = false;
                if(table.values.length > 0 && table.columns.length < table.values[0].length) {
                    colShiftFlag = true;
                }

                
                for (var i = colsToDelete.length - 1; i >= 0; i--) {
                    table.columns.splice(colsToDelete[i], 1);
                    table.values.forEach(function (value) {
                        if(colShiftFlag) {
                            value.splice(colsToDelete[i] + 1, 1);
                        } else {
                            value.splice(colsToDelete[i], 1);
                        }
                    });
                }
            });
        },

        _hideColumnsInitially: function (columnsArray, columnsToHideArray) {
            array.forEach(columnsToHideArray, function (columnToHide) {
                console.log(columnToHide);
            });
        },

        _getPaginationMixin: function () {
            var i18nCustomized = lang.mixin({}, i18nPagination, {
                status: I18nUtil.get('pagination_status'),
                gotoFirst: I18nUtil.get('pagination_first'),
                gotoLast: I18nUtil.get('pagination_last'),
                gotoNext: I18nUtil.get('pagination_next'),
                gotoPrev: I18nUtil.get('pagination_previous'),
                gotoPage: I18nUtil.get('pagination_page'),
                jumpPage: I18nUtil.get('pagination_jump')
            });
            var MyPagination = declare(Pagination, {
                i18nPagination: i18nCustomized
            });

            return MyPagination;
        },

        _checkAllButtonClicked: function (e, column, checkbox, table) {
            e.stopPropagation();
            var checkboxes = query('.dgrid-scroller input[type=checkbox]');
            array.forEach(checkboxes, function (entityNode) {
                var entity = registry.byId(entityNode.id);
                entity.set('checked', checkbox.get('checked'));
            });
            if(table._oncheck){
                table._oncheck(e); 
            }
        },

        
        render: function (table, d_target, disableStartup, handlers, tableOptions, filterFunction) {
            var dataGenerator = objectModelUtil.getDataGenerator(table, this._rawData);
            var columns = dataGenerator.columns();
            var rowsStore = dataGenerator.rowsStore();
            var that = this;

            if (typeof(dataGenerator.getRawTableData().errorTable) !== 'undefined' && dataGenerator.getRawTableData().errorTable === true) {
                this.renderErrorTable(dataGenerator.rows(), d_target);
                
                return {};
            }

            var options = lang.mixin({}, this._options);
            options = lang.mixin(options, tableOptions);


            
            
            
            if (options.initialHiddenColumns && options.initialHiddenColumns.length > 0) {
                this._hideColumnsInitially(columns, options.initialHiddenColumns);
            }

            
            
            var ColumnHiderFix = declare(null, {});

            var CustomHider = declare([ColumnHider], {
                _renderHiderMenuEntry: function (col) {
                    
                    
                    var oldLabel = col.label;
                    col.label = domConstruct.toDom('<span>%0</span>'.tokenize(col.label)).innerHTML;

                    this.inherited(arguments);

                    col.label = oldLabel;
                },
                _updateColumnHiddenState: function (){
                    that._options.customView.clearCustomViewIndicator();
                    this.inherited(arguments);
                }
            });

            
            
            
            
            var baseOptions = [Grid, MultiColumnSortingMixin, DijitRegistry, this._getPaginationMixin(),
                ColumnResizer, Keyboard, CustomHider, ColumnHiderFix, ColumnReorderWidthPreserveMixin];
            if (options.enableSelection) {
                baseOptions.push(Selection);
            }

            var GridWidget = declare(baseOptions);
            var checkBoxWidth = '25px';

            var selectionMode = 'multiple';
            if (options.useRadio) {
                selectionMode = 'single';
            }
            else if (options.selectionMode) {
                selectionMode = options.selectionMode;
            }

            var measuresTable = this;

            
            if (options.useCheckboxes || options.useRadio) {
                columns.splice(0, 0, {
                    field: '',
                    label: ' ',
                    unhidable: true,
                    reorderable: false,
                    renderHeaderCell: function (node) {
                        var d_checkbox = (options.useCheckboxes) ? new CheckBox() : new RadioButton();
                        style.set(node, 'width', checkBoxWidth);
                        var that = this;
                        on(d_checkbox, 'click', function (e) {
                            measuresTable._checkAllButtonClicked(e, that, this, measuresTable);
                        });
                        this.grid.checkAllCheckbox = d_checkbox;
                        return d_checkbox.domNode;
                    },
                    renderCell: function (object, value, node) {
                        var d_checkbox;
                        if (options.useCheckboxes) {
                            d_checkbox = new CheckBox();
                        }
                        else {
                            d_checkbox = new RadioButton();
                        }

                        d_checkbox.startup();

                        
                        on(node, 'dblclick', function (e) {
                            e.stopPropagation();
                        });

                        on(d_checkbox, 'click', function (e) {
                            e.stopPropagation();

                            if (that._oncheck) {
                                that._oncheck(e, object, d_checkbox);
                            }

                            if (handlers && handlers.oncheck) {
                                handlers.oncheck(gridWidget.row(e), e, object, d_checkbox);
                            }
                        });
                        on(d_checkbox, 'change', function (e) {
                            if (that._onchange) {
                                that._onchange(e, object, d_checkbox);
                            }
                        });

                        style.set(node, 'width', checkBoxWidth);

                        
                        
                        this.grid.itemsToDestroy = this.grid.itemsToDestroy || [];
                        this.grid.itemsToDestroy.push(d_checkbox);
                        return d_checkbox.domNode;
                    }
                });
            }

            var pageSizes = getPageSizes(rowsStore.data.length);
            var predeterminedPageSize = 0;

            
            if (options.rowsPerPage === -1) {
                predeterminedPageSize = pageSizes[pageSizes.length - 1];
            }
            else {
                predeterminedPageSize = options.rowsPerPage || pageSizes[1] || pageSizes[0];
            }

            if(options.hidePageSelector){
                pageSizes = "";
            }

            var gridWidget = new GridWidget({
                name: table,
                columns: columns,
                store: rowsStore,
                rowsPerPage: predeterminedPageSize, 
                query: filterFunction,
                selectionMode: selectionMode,
                pageSizeOptions: pageSizes,
                previousNextArrows: !options.hidePageSelector,
                _resizeMouseUp: function (){
                    that._options && that._options.customView && that._options.customView.clearCustomViewIndicator();
                    this.inherited(arguments);
                },
                clearCustomViewIndicator: function(){
                    that._options && that._options.customView && that._options.customView.clearCustomViewIndicator();
                },
                
                buildRendering: function () {
                    this.inherited(arguments);

                    var linkNodes = query('a[href^="javascript:"]');
                    array.forEach(linkNodes, function (node) {
                        domAttr.remove(node, 'href');
                        on(node, 'click', function () {
                            return false;
                        });
                    });
                },
                _updateNavigation: function (focusLink) {
                    this.inherited(arguments);

                    if (that.updateCheckAllBox) {
                        
                        that.updateCheckAllBox();
                    }

                    if (that.resetDispostionList) {
                        that.resetDispostionList();
                    }

                    var linkNodes = query('a[href^="javascript:"]');
                    array.forEach(linkNodes, function (node) {
                        domAttr.remove(node, 'href');
                        on(node, 'click', function () {
                            return false;
                        });
                    });
                },
                
                destroy: function () {
                    this.inherited(arguments);

                    array.forEach(this.itemsToDestroy, function (item) {
                        if (item.destroy) {
                            item.destroy();
                        }
                    });
                    window.itemsToDestroy = [];
                },
                _setRowsPerPage: function (rowsPerPage) {
                    
                    if (rowsPerPage && !isNaN(Number(rowsPerPage))) {
                        this.rowsPerPage = Number(rowsPerPage);
                    }
                    this._updateRowsPerPageOption();
                    this.gotoPage(1);

                },
                gotoPage: function (page, focusLink) {
                    var promise = this.inherited(arguments);

                    if (handlers && handlers.onPageChange) {
                        handlers.onPageChange(page);
                    }
                    return promise;
                },

                
                _move: function(item, steps, targetClass, visible){
                    var nextSibling, current, element;
                    
                    element = current = item.element;
                    steps = steps || 1;
                    do{
                        
                        if((nextSibling = current[steps < 0 ? "previousSibling" : "nextSibling"])){
                            do{
                                
                                current = nextSibling;
                                
                                if(current.columnId) current.hidden = this.columns[Number(current.columnId)].hidden;
                                if(current && !current.hidden && (current.className + " ").indexOf(targetClass + " ") > -1){
                                    
                                    element = current;
                                    steps += steps < 0 ? 1 : -1;
                                    break;
                                }
                                
                                
                            }while((nextSibling = (!visible || !current.hidden) && current[steps < 0 ? "lastChild" : "firstChild"]));
                        }else{
                            current = current.parentNode;
                            if(current === this.bodyNode || current === this.headerNode){
                                
                                break;
                            }
                        }
                    }while(steps);
                    
                    
                    return element;
                },



                
                
                _moveColumn: function (myArray, from, to) {
                    myArray.splice(to, 0, myArray.splice(from, 1)[0]);
                },

                
                
                _findSubRowIndex: function (value, startIndex) {
                    var columnList = this.subRows[0];
                    for (var i = startIndex; i < columnList.length; i++) {
                        if (columnList[i].field == value) {
                            return i;
                        }
                    }
                    return -1;
                },


                
                setView: function (columnPreferences, columnHiding) {
                    if (columnPreferences) {
                        this.applyPreferencesToSubrows(columnPreferences.columns);
                        this.set("sort", columnPreferences.sort);
                    }
                    var that = this;
                    array.forEach(columnHiding, function (columnIndex) {
                        var currentSubRow = that.subRows[0][columnIndex];
                        currentSubRow.unhidable = true;
                        currentSubRow.hidden = true;
                    });
                    that.set("subRows", that.subRows);

                },


                makeUnhidable: function (columnHiding) {
                    var that = this;
                    array.forEach(columnHiding, function (columnIndex) {
                        var currentSubRow = that.subRows[0][columnIndex];
                        currentSubRow.unhidable = true;
                        currentSubRow.hidden = true;
                    });
                    that.set("subRows", that.subRows);
                },

                setGridPreferences: function (columnPreferences) {
                    if (!columnPreferences) return 0;
                    var numMatchingPreferences = this.applyPreferencesToSubrows(columnPreferences.columns);
                    if (numMatchingPreferences > 0) {
                        this.set("subRows", this.subRows);
                        this.set("sort", columnPreferences.sort);
                    }
                    return numMatchingPreferences;
                },

                gridPreferences: function () {
                    return GridPrefs.getGridPrefs(this);
                },

                
                applyPreferencesToSubrows: function (preferences) {
                    var currentGridColumn = 0;
                    
                    for (var i = 0; i < preferences.length; i++) {
                        
                        var foundLocation = this._findSubRowIndex(preferences[i].id, currentGridColumn);
                        
                        if (foundLocation < 0) {
                            continue;
                        }
                        
                        if (foundLocation != currentGridColumn) {
                            this._moveColumn(this.subRows[0], foundLocation, currentGridColumn);
                        }
                        
                        var currentSubRow = this.subRows[0][currentGridColumn];
                        
                        
                        if(preferences[i].w) currentSubRow.width = preferences[i].w;

                        
                        
                        
                        
                        
                        
                        if (!currentSubRow.unhidable) {
                            currentSubRow.hidden = preferences[i].hidden;
                        }
                        currentGridColumn++;
                    }
                    return currentGridColumn;
                },

                
                
                turnOffSortAndOrder: function () {
                    for (var i = 0; i < this.subRows[0].length; i++) {
                        this.subRows[0][i].reorderable = false;
                        this.subRows[0][i].sortable = false;
                    }
                },


                
                
                deleteGridPrefs: function (name) {
                    GridPrefs.deleteGridPrefs(name);
                },
                
                deleteAllGridPrefs: function () {
                    GridPrefs.deleteAllGridPrefs();
                },

                renderArray: function (results, beforeNode) {
                    var rowNodes = this.inherited(arguments);

                    var calculationFunction = function () {
                        if(rowNodes[rowNodes.length -1].parentElement == null) return 0;
                        return domGeometry.position(rowNodes[rowNodes.length - 1]).y - domGeometry.position(rowNodes[0]).y + domGeometry.position(rowNodes[rowNodes.length - 1]).h;
                    };

                    var that = this;
                    var renderCompletionFunction = function () {
                        if (options.calculateHeight) {
                            var height = calculationFunction();
                            if (height === 0) {
                                setTimeout(renderCompletionFunction, 100);
                            }
                            else {
                                that.calculatedHeight = height;

                                if (options.renderComplete) {
                                    options.renderComplete(that);
                                }

                            }
                        }
                    };
                    renderCompletionFunction();
                    return rowNodes;
                }
            }, d_target);

            gridWidget.declaredClass = 'dgrid';
            gridWidget.addKeyHandler(32,function (event) {
                if (handlers.ondblclick) {
                    handlers.ondblclick(gridWidget.row(event));
                }
            });

            on(gridWidget.hiderToggleNode, 'mousedown', function (e) {
                e.stopPropagation();
            });

            
            domAttr.set(gridWidget.domNode, 'data-table-name', table);

            var paginationNode = gridWidget.paginationNode;

            gridWidget.setRowsPerPage = function (rowsPerPage) {
                
                var pageSelector = query('select.dgrid-page-size', paginationNode)[0];
                if(pageSelector) pageSelector.value = rowsPerPage;
            };
            gridWidget.setRowsPerPage(predeterminedPageSize);

            if (options.gridHeight) {
                domStyle.set(gridWidget.domNode, 'height', options.gridHeight);
            }

            
            if (!disableStartup) {
                gridWidget.startup();
            }

            marked.setOptions({
                renderer: new marked.Renderer(),
                gfm: true, 
                tables: true, 
                breaks: true, 
                pedantic: false,
                sanitize: true, 
                smartLists: true,
                smartypants: false
            });

            
            on(gridWidget.domNode, on.selector(".dgrid-cell", mouse.enter), function(event) {
                var cell = gridWidget.cell(this);
                var timeout = 500; 
                
                if (cell.column.field && cell.row && cell.row.id && that.tooltips[gridWidget.name][cell.column.field]) {
                    
                    clearTimeout(gridWidget.mouseLeaveTooltipTimeoutID);
                    clearTimeout(gridWidget.mouseLeaveCellTimeoutID);
                    var cellTooltipData = that.tooltips[gridWidget.name][cell.column.field];
                    if (cellTooltipData[cell.row.id]) {
                        
                        cell.mouseLeaveSignal = on.once(this, mouse.leave, function () {
                            gridWidget.mouseLeaveCellTimeoutID = setTimeout(function() {
                                Tooltip.hide(cell.element);
                            }, timeout);
                        });

                        var markedTooltipData = "";
                        if(cellTooltipData[cell.row.id].length >= 2) {
                            markedTooltipData = marked(cellTooltipData[cell.row.id][1]);
                        } else {
                            
                            markedTooltipData = marked(cellTooltipData[cell.row.id][0]);
                        }

                        if (markedTooltipData.length > 0) {
                            Tooltip.show(markedTooltipData, this, null, null, null, function () {
                                
                                
                                
                                clearTimeout(gridWidget.mouseLeaveCellTimeoutID);
                                clearTimeout(gridWidget.mouseLeaveTooltipTimeoutID);
                            }, function () { 
                                
                                gridWidget.mouseLeaveTooltipTimeoutID = setTimeout(function () {
                                    Tooltip.hide(cell.element);
                                }, timeout);
                            });
                        }
                    }
                }
            });

            gridWidget.on(".dgrid-row:dblclick", function (event) {
                if (handlers.ondblclick) {
                    handlers.ondblclick(gridWidget.row(event));
                }
            });

            gridWidget.on(touch.selector(".dgrid-row:dblclick", touch.dbltap), function (event) {
                if (handlers.ondblclick) {
                    handlers.ondblclick(gridWidget.row(event));
                }
            });

            gridWidget.on(".dgrid-row:click", function (event) {
                if (handlers.onclick) {
                    handlers.onclick(gridWidget.row(event));
                }
            });

            gridWidget.on('dgrid-columnstatechange', function (event) {
                if (handlers.onColumnStateChange) {
                    handlers.onColumnStateChange(event.column.field, event.hidden);
                }
            });

            
            if (options.selectFirst) {
                gridWidget.select(query('.dgrid-row', gridWidget.domNode)[0]);
            }

            gridWidget.dataGenerator = dataGenerator;

            return gridWidget;
        },

        renderErrorTable: function (rows, target) {
            require([
                'sna/util/I18nUtil',
                'dojo/text!src/sna/resources/html/templates/errortable.html'
            ], function (I18nUtil, errorTableHtml) {
                target.innerHTML = errorTableHtml;

                var errorContentNode = query('[data-role=errorContent]', target)[0];
                
                errorContentNode.innerHTML = rows[0][0][1];
                
                var errorDetailsNode = query('[data-role=errorDetails]', target)[0];

                if (rows[1][0] && rows[1][0][1]) {
                    errorDetailsNode.innerHTML = rows[1][0][1];
                }
            })
        }
    });
});

               