
                   
define([
    'dojo/dom-construct',
    'dojo/_base/declare'
], function( domConstruct, declare ) {
    return declare(null, {
        
        constructor: function(args) {
            this._content = '';

            var width = '';

            if(args.width) {
                width = 'style = "width: ' + args.width + ';"';
            }

            this._table = {
                domNode: domConstruct.toDom('<table %0></table>'.tokenize(width)),
                rows: []
            };

            var i = 0;
            if(args.rows && args.columns) {
                for(i = 0; i < args.rows; i += 1) {
                    this.addRow(args.columns);
                }
            }

            this.maxColumns = args.maxColumns || 0;
        },

        
        getTable: function() {
            return this._table;
        },

        
        addRow: function(columns) {
            columns = columns || 0;

            var element = domConstruct.toDom('<tr></tr>');
            var i = 0;
            var columnObjects = [];

            for(i =0; i < columns; i += 1) {
                var td = domConstruct.toDom('<td></td>');
                domConstruct.place(td, element);
                columnObjects.push({
                    domNode: td
                });
            }

            domConstruct.place(element, this._table.domNode);
            var rowObject = {
                domNode: element,
                columns: columnObjects
            };

            this._table.rows.push(rowObject);

            return rowObject;
        },

        
        addColumn: function(row, element, inPlace, classLabel){
            var td;
            if(classLabel){
                td = domConstruct.toDom('<td class="'+ classLabel + '"></td>');
            }else{
                td = domConstruct.toDom('<td></td>');
            }

            if(inPlace) {
                td = element;
            }

            if(element && !inPlace) {
                domConstruct.place(element, td);
            }

            var columnObject = {
                domNode: td
            };

            var newRow = row;

            if(this.maxColumns > 0 && row.columns.length >= this.maxColumns) {
                newRow = this.addRow(0);
            }

            domConstruct.place(td, newRow.domNode);
            newRow.columns.push(columnObject);
            return {
                column: columnObject,
                row: newRow
            };
        },

        
        placeElement: function(element, label, placement) {
            if(this._table.rows.length !== 2 || this._table.rows[0].columns.length !== 2) {
                console.error("can't place element because table is not 2x2", placement, this._table);
                return;
            }

            if(placement === 'left') {
                domConstruct.place(label, this._table.rows[0].columns[0].domNode);
                domConstruct.place(element, this._table.rows[0].columns[1].domNode);
            }
            else if (placement === 'top') {
                domConstruct.place(label, this._table.rows[0].columns[0].domNode);
                domConstruct.place(element, this._table.rows[1].columns[0].domNode);
            }
            else if (placement === 'right') {
                domConstruct.place(label, this._table.rows[0].columns[1].domNode);
                domConstruct.place(element, this._table.rows[0].columns[0].domNode);
            }
            else if (placement === 'bottom') {
                domConstruct.place(label, this._table.rows[1].columns[0].domNode);
                domConstruct.place(element, this._table.rows[0].columns[0].domNode);
            }
        },
        
        
        destroy: function() {
        	domConstruct.destroy(this._table.domNode);
        }
    });
});

               